package org.ow2.util.ee.metadata.ejbjar.impl.configurator;

import java.util.List;
import java.util.Set;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.common.impl.configurator.CommonClassMetadataConfigurator;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxAnnotationSecurityPermitAllVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxAnnotationSecurityRolesAllowedVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxEjbTransactionAttributeVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxInterceptorExcludeDefaultInterceptorsVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxInterceptorInterceptorsVisitor;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxAnnotationSecurityDeclareRolesVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxAnnotationSecurityRunAs;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbApplicationExceptionVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbLocalHomeVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbLocalVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbMessageDrivenVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbRemoteHomeVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbRemoteVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbStatefulVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbStatelessVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbTransactionManagementVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.specific.ISpecificEjbJarDeployableMetadataConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.metadata.structures.JClass;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;


/**
 * EjbJarClassMetadataConfigurator.
 * @param <E> deployable type
 * @author Gael Lalire
 */
public class EjbJarClassMetadataConfigurator<E extends EJBDeployable<E>> extends
        CommonClassMetadataConfigurator<IEjbJarClassMetadata<E>, IEjbJarMethodMetadata<E>, IEjbJarFieldMetadata<E>> {


    /**
     * annotationParsingDesactived.
     */
    private boolean annotationParsingDesactived;

    /**
     * ejbJarArchiveMetadata.
     */
    private EjbJarDeployableMetadata<E> ejbJarArchiveMetadata;

    /**
     * EjbJarClassMetadataConfigurator.
     * @param jClass signature
     * @param ejbJarArchiveMetadata parent
     * @param annotationParsingDesactived annotationParsingDesactived
     * @param specificEjbJarDeployableFactoryList list of specific metadata factories
     */
    public EjbJarClassMetadataConfigurator(final JClass jClass, final EjbJarDeployableMetadata<E> ejbJarArchiveMetadata,
            final boolean annotationParsingDesactived,
            final List<ISpecificEjbJarDeployableMetadataConfigurator<E>> specificEjbJarDeployableFactoryList) {
        super(new EjbJarClassMetadata<E>(jClass, ejbJarArchiveMetadata), specificEjbJarDeployableFactoryList);
        this.ejbJarArchiveMetadata = ejbJarArchiveMetadata;
        this.annotationParsingDesactived = annotationParsingDesactived;
        if (!annotationParsingDesactived) {
            registerAnnotationVisitor();
        }
    }

    /**
     * register annotation visitor.
     */
    @Override
    protected void registerAnnotationVisitor() {

        super.registerAnnotationVisitor();

        // TODO gaellalire : use interface in visitors
        EjbJarClassMetadata<E> ejbJarClassMetadata = (EjbJarClassMetadata<E>) getClassMetadata();

        // add @Local
        getAnnotationVisitors().put(JavaxEjbLocalVisitor.TYPE, new JavaxEjbLocalVisitor<E>(ejbJarClassMetadata));

        // add @Remote
        getAnnotationVisitors().put(JavaxEjbRemoteVisitor.TYPE, new JavaxEjbRemoteVisitor<E>(ejbJarClassMetadata));

        // add @Stateless
        getAnnotationVisitors().put(JavaxEjbStatelessVisitor.TYPE,
                new JavaxEjbStatelessVisitor<E>(ejbJarClassMetadata));

        // add @Stateful
        getAnnotationVisitors().put(JavaxEjbStatefulVisitor.TYPE, new JavaxEjbStatefulVisitor<E>(ejbJarClassMetadata));

        // add @MessageDriven
        getAnnotationVisitors().put(JavaxEjbMessageDrivenVisitor.TYPE, new JavaxEjbMessageDrivenVisitor<E>(
                ejbJarClassMetadata));

        // add @LocalHome
        getAnnotationVisitors().put(JavaxEjbLocalHomeVisitor.TYPE,
                new JavaxEjbLocalHomeVisitor<E>(ejbJarClassMetadata));

        // add @RemoteHome
        getAnnotationVisitors().put(JavaxEjbRemoteHomeVisitor.TYPE, new JavaxEjbRemoteHomeVisitor<E>(
                ejbJarClassMetadata));

        // add @TransactionManagement
        getAnnotationVisitors().put(JavaxEjbTransactionManagementVisitor.TYPE,
                new JavaxEjbTransactionManagementVisitor<E>(ejbJarClassMetadata));

        // add @TransactionAttribute
        getAnnotationVisitors().put(JavaxEjbTransactionAttributeVisitor.TYPE,
                new JavaxEjbTransactionAttributeVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

        // add @Interceptors
        getAnnotationVisitors().put(JavaxInterceptorInterceptorsVisitor.TYPE,
                new JavaxInterceptorInterceptorsVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

        // add @ApplicationException
        getAnnotationVisitors().put(JavaxEjbApplicationExceptionVisitor.TYPE, new JavaxEjbApplicationExceptionVisitor<E>(
                ejbJarClassMetadata));

        // add @DeclareRoles
        getAnnotationVisitors().put(JavaxAnnotationSecurityDeclareRolesVisitor.TYPE,
                new JavaxAnnotationSecurityDeclareRolesVisitor<E>(ejbJarClassMetadata));

        // add @RolesAllowed
        getAnnotationVisitors().put(JavaxAnnotationSecurityRolesAllowedVisitor.TYPE,
                new JavaxAnnotationSecurityRolesAllowedVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

        // add @PermitAll
        getAnnotationVisitors().put(JavaxAnnotationSecurityPermitAllVisitor.TYPE,
                new JavaxAnnotationSecurityPermitAllVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

        // add @RunAs
        getAnnotationVisitors().put(JavaxAnnotationSecurityRunAs.TYPE,
                new JavaxAnnotationSecurityRunAs<E>(ejbJarClassMetadata));

        // add @ExcludeDefaultInterceptors
        getAnnotationVisitors().put(JavaxInterceptorExcludeDefaultInterceptorsVisitor.TYPE,
                new JavaxInterceptorExcludeDefaultInterceptorsVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

    }

    /**
     * @param jField signature
     * @return configurator
     */
    @Override
    protected IFieldConfigurator createFieldMetadataConfigurator(final JField jField) {
        return new EjbJarFieldMetadataConfigurator<E>(jField, getClassMetadata(), this.annotationParsingDesactived,
                getSpecificClassConfiguratorList());
    }

    /**
     * @param jMethod signature
     * @return configurator
     */
    @Override
    protected IMethodConfigurator createMethodMetadataConfigurator(final JMethod jMethod) {
        return new EjbJarMethodMetadataConfigurator<E>(jMethod, getClassMetadata(), this.annotationParsingDesactived,
                getSpecificClassConfiguratorList());
    }

    /**
     * add class to archive.
     * @param referencedClasses you may fill this set with class referenced in current metadata
     */
    @Override
    public void configurationComplete(final Set<String> referencedClasses) {
       IEjbJarClassMetadata<E> ejbJarClassMetadata = getClassMetadata();
       ejbJarArchiveMetadata.addEjbJarClassMetadata(ejbJarClassMetadata);
       // add interceptors class name on class
       IJInterceptors interceptors = ejbJarClassMetadata.getAnnotationInterceptors();
       if (interceptors != null) {
           referencedClasses.addAll(interceptors.getClasses());
       }
       for (IEjbJarMethodMetadata<?> methodMetadata : ejbJarClassMetadata.getStandardMethodMetadataCollection()) {
           // add interceptors class name on method
           interceptors = methodMetadata.getAnnotationInterceptors();
           if (interceptors != null) {
               referencedClasses.addAll(interceptors.getClasses());
           }
       }
       super.configurationComplete(referencedClasses);
    }

}
