package org.ow2.util.ee.metadata.ejbjar.impl.configurator;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.specific.ISpecificEjbJarDeployableMetadataConfigurator;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.metadata.structures.JClass;

/**
 * EjbJarArchiveMetadataConfigurator.
 * @param <E> deployable type
 * @author Gael Lalire
 */
public class EjbJarDeployableMetadataConfigurator<E extends EJBDeployable<E>> implements IArchiveConfigurator {

    private List<ISpecificEjbJarDeployableMetadataConfigurator<E>> specificEjbJarDeployableFactoryList;

    /**
     * archive metadata.
     */
    private EjbJarDeployableMetadata<E> ejbJarArchiveMetadata;

    /**
     * annotationParsingDesactived.
     */
    private boolean annotationParsingDesactived;

    /**
     * Constructor.
     * @param ejbDeployable the deployable
     */
    public EjbJarDeployableMetadataConfigurator(final E ejbDeployable) {
        this.ejbJarArchiveMetadata = new EjbJarDeployableMetadata<E>(ejbDeployable);
        this.annotationParsingDesactived = false;
        specificEjbJarDeployableFactoryList = new ArrayList<ISpecificEjbJarDeployableMetadataConfigurator<E>>();
    }

    /**
     * @param jClass signature
     * @return class configurator
     */
    public IClassConfigurator createClassConfigurator(final JClass jClass) {
        return new EjbJarClassMetadataConfigurator<E>(jClass, this.ejbJarArchiveMetadata, this.annotationParsingDesactived, specificEjbJarDeployableFactoryList);
    }

    /**
     * @return the archive metadata
     */
    public EjbJarDeployableMetadata<E> getEjbJarArchiveMetadata() {
        return this.ejbJarArchiveMetadata;
    }

    /**
     * @return true if annotation parsing is desactived
     */
    public boolean isAnnotationParsingDesactived() {
        return this.annotationParsingDesactived;
    }

    /**
     * Set annotationParsingDesactived.
     * @param annotationParsingDesactived true if you want to desactive annotation parsing
     */
    public void setAnnotationParsingDesactived(final boolean annotationParsingDesactived) {
        this.annotationParsingDesactived = annotationParsingDesactived;
    }

    public void addSpecificMetadataConfigurator(final ISpecificEjbJarDeployableMetadataConfigurator<E> specificEjbJarDeployableFactory) {
        specificEjbJarDeployableFactoryList.add(specificEjbJarDeployableFactory);
    }

    public List<ISpecificEjbJarDeployableMetadataConfigurator<E>> getSpecificEjbJarDeployableFactoryList() {
        return specificEjbJarDeployableFactoryList;
    }

}
