/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbsCommonEjbVisitor.java 3221 2008-05-15 08:49:06Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AbsAnnotationVisitor;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarClassMetadata;

/**
 * This class manages the handling of common annotations used by beans.
 * @param <T> a class extending JCommonBean.
 * @param <E> deployable type
 * @author Florent Benoit
 */
public abstract class AbsCommonEjbVisitor<T extends IJCommonBean, E extends EJBDeployable<E>> extends
        AbsAnnotationVisitor<EjbJarClassMetadata<E>> {

    /**
     * Name attribute of the annotation.
     */
    private static final String NAME = "name";

    /**
     * mappedName attribute of the annotation.
     */
    private static final String MAPPED_NAME = "mappedName";

    /**
     * description attribute of the annotation.
     */
    private static final String DESCRIPTION = "description";

     /**
     * Constructor.
     * @param classAnnotationMetadata linked to a class metadata
     */
    public AbsCommonEjbVisitor(final EjbJarClassMetadata<E> classAnnotationMetadata) {
        super(classAnnotationMetadata);
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (name.equals(NAME)) {
            getJCommonBean().setName((String) value);
        } else if (name.equals(MAPPED_NAME)) {
            getJCommonBean().setMappedName((String) value);
        } else if (name.equals(DESCRIPTION)) {
            getJCommonBean().setDescription((String) value);
        }
    }

    /**
     * Visits the end of the annotation. <br> Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        getAnnotationMetadata().setJCommonBean(getJCommonBean());
    }

    /**
     * @return the object used by all beans.
     */
    public abstract T getJCommonBean();


}
