/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxEjbRemoteHomeVisitor.java 3285 2008-05-22 09:30:26Z gaellalire $
 * --------------------------------------------------------------------------
 */


package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AnnotationType;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.ObjectAnnotationVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarClassMetadata;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.ejb.RemoteHome} annotation.
 * @param <E> deployable type
 * @author Florent Benoit
 */
public class JavaxEjbRemoteHomeVisitor<E extends EJBDeployable<E>>
        extends ObjectAnnotationVisitor<EjbJarClassMetadata<E>, IType> implements AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/RemoteHome;";

    /**
     * Constructor.
     * @param classAnnotationMetadata linked to a class metadata.
     */
    public JavaxEjbRemoteHomeVisitor(final EjbJarClassMetadata<E> classAnnotationMetadata) {
        super(classAnnotationMetadata);
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        IType t = getValue();
        String className = t.getInternalName();
        getAnnotationMetadata().setRemoteHome(className);
    }

    /**
     * @return type of the annotation (its description).
     */
    public String getType() {
        return TYPE;
    }

}
