/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxEjbStatefulVisitor.java 3319 2008-05-27 13:00:31Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz;

import static org.ow2.util.ee.metadata.ejbjar.api.ClassType.STATEFUL;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AnnotationType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateful;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JStateful;

/**
 * This class manages the handling of &#64;{@link javax.ejb.Stateful}
 * annotation.
 * @param <E> deployable type
 * @author Florent Benoit
 */
public class JavaxEjbStatefulVisitor<E extends EJBDeployable<E>> extends AbsCommonEjbVisitor<IJStateful, E> implements
        AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/Stateful;";

    /**
     * Constructor.
     * @param classAnnotationMetadata linked to a class metadata
     */
    public JavaxEjbStatefulVisitor(final EjbJarClassMetadata<E> classAnnotationMetadata) {
        super(classAnnotationMetadata);
    }

    /**
     * Visits the end of the annotation. <br> Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        super.visitEnd();
        getAnnotationMetadata().setClassType(STATEFUL);
    }

    /**
     * @return the object representing common bean.
     */
    @Override
    public IJStateful getJCommonBean() {
        IJStateful jStateful = getAnnotationMetadata().getJStateful();
        if (jStateful == null) {
            jStateful = new JStateful();
            getAnnotationMetadata().setJStateful(jStateful);
        }
        return jStateful;
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }

}
