/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: SpecificEjbJarClassMetadata.java 3524 2008-06-20 13:37:06Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.specific;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagementType;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.ejbjar.api.ClassType;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IEnvEntry;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJLocal;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJRemote;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateful;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateless;
import org.ow2.util.scan.api.metadata.specific.ISpecificClassMetadata;
import org.ow2.util.scan.api.metadata.specific.ISpecificFieldMetadata;
import org.ow2.util.scan.api.metadata.specific.ISpecificMethodMetadata;
import org.ow2.util.scan.api.metadata.specific.SpecificClassMetadata;
import org.ow2.util.scan.api.metadata.structures.JClass;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * You can extends this class to create a specific war class metadata.
 * @param <E> deployable type
 * @param <SC> specific class metadata
 * @param <SM> specific method metadata
 * @param <SF> specific field metadata
 * @author Gael Lalire
 */
public class SpecificEjbJarClassMetadata<
    E extends EJBDeployable<E>,
    SC extends ISpecificClassMetadata<SC, SM, SF>,
    SM extends ISpecificMethodMetadata<SC, SM, SF>,
    SF extends ISpecificFieldMetadata<SC, SM, SF>>
        extends SpecificClassMetadata<SC, SM, SF> implements IEjbJarClassMetadata<E> {

    /**
     * The standard war class metadata.
     */
    private IEjbJarClassMetadata<E> ejbJarClassMetadata;

    /**
     * Construct delegation.
     * @param ejbJarClassMetadata the standard ejb jar class metadata.
     */
    protected SpecificEjbJarClassMetadata(final IEjbJarClassMetadata<E> ejbJarClassMetadata) {
        this.ejbJarClassMetadata = ejbJarClassMetadata;
    }

    /**
     * {@inheritDoc}
     */
    public JClass getJClass() {
        return ejbJarClassMetadata.getJClass();
    }

    /**
     * {@inheritDoc}
     */
    public void setJClass(final JClass class1) {
        ejbJarClassMetadata.setJClass(class1);
    }

    /**
     * {@inheritDoc}
     */
    public List<IJAnnotationResource> getJAnnotationResources() {
        return ejbJarClassMetadata.getJAnnotationResources();
    }

    /**
     * {@inheritDoc}
     */
    public List<IJEjbEJB> getJEjbEJBs() {
        return ejbJarClassMetadata.getJEjbEJBs();
    }

    /**
     * {@inheritDoc}
     */
    public List<IJavaxPersistenceContext> getJavaxPersistencePersistenceContexts() {
        return ejbJarClassMetadata.getJavaxPersistencePersistenceContexts();
    }

    /**
     * {@inheritDoc}
     */
    public List<IJavaxPersistenceUnit> getJavaxPersistencePersistenceUnits() {
        return ejbJarClassMetadata.getJavaxPersistencePersistenceUnits();
    }

    /**
     * {@inheritDoc}
     */
    public void setJAnnotationResources(final List<IJAnnotationResource> annotationResources) {
        ejbJarClassMetadata.setJAnnotationResources(annotationResources);
    }

    /**
     * {@inheritDoc}
     */
    public void setJEjbEJBs(final List<IJEjbEJB> ejbEJBs) {
        ejbJarClassMetadata.setJEjbEJBs(ejbEJBs);
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaxPersistencePersistenceContexts(final List<IJavaxPersistenceContext> javaxPersistencePersistenceContexts) {
        ejbJarClassMetadata.setJavaxPersistencePersistenceContexts(javaxPersistencePersistenceContexts);
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaxPersistencePersistenceUnits(final List<IJavaxPersistenceUnit> javaxPersistencePersistenceUnits) {
        ejbJarClassMetadata.setJavaxPersistencePersistenceUnits(javaxPersistencePersistenceUnits);
    }

    /**
     * {@inheritDoc}
     */
    public IJEjbEJB getJEjbEJB() {
        return ejbJarClassMetadata.getJEjbEJB();
    }

    /**
     * {@inheritDoc}
     */
    public void setJEjbEJB(final IJEjbEJB ejbEJB) {
        ejbJarClassMetadata.setJEjbEJB(ejbEJB);
    }

    /**
     * {@inheritDoc}
     */
    public IJAnnotationResource getJAnnotationResource() {
        return ejbJarClassMetadata.getJAnnotationResource();
    }

    /**
     * {@inheritDoc}
     */
    public void setJAnnotationResource(final IJAnnotationResource annotationResource) {
        ejbJarClassMetadata.setJAnnotationResource(annotationResource);
    }

    /**
     * {@inheritDoc}
     */
    public IJavaxPersistenceContext getJavaxPersistenceContext() {
        return ejbJarClassMetadata.getJavaxPersistenceContext();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPersistenceContext() {
        return ejbJarClassMetadata.isPersistenceContext();
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaxPersistenceContext(final IJavaxPersistenceContext javaxPersistenceContext) {
        ejbJarClassMetadata.setJavaxPersistenceContext(javaxPersistenceContext);
    }

    /**
     * {@inheritDoc}
     */
    public IJavaxPersistenceUnit getJavaxPersistenceUnit() {
        return ejbJarClassMetadata.getJavaxPersistenceUnit();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPersistenceUnit() {
        return ejbJarClassMetadata.isPersistenceUnit();
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaxPersistenceUnit(final IJavaxPersistenceUnit javaxPersistenceUnit) {
        ejbJarClassMetadata.setJavaxPersistenceUnit(javaxPersistenceUnit);
    }

    /**
     * {@inheritDoc}
     */
    public IJInterceptors getAnnotationInterceptors() {
        return ejbJarClassMetadata.getAnnotationInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public ApplicationException getApplicationException() {
        return ejbJarClassMetadata.getApplicationException();
    }

    /**
     * {@inheritDoc}
     */
    public String getClassName() {
        return ejbJarClassMetadata.getClassName();
    }

    /**
     * {@inheritDoc}
     */
    public List<String> getDeclareRoles() {
        return ejbJarClassMetadata.getDeclareRoles();
    }

    /**
     * {@inheritDoc}
     */
    public IEjbJarDeployableMetadata<E> getEjbJarDeployableMetadata() {
        return ejbJarClassMetadata.getEjbJarDeployableMetadata();
    }

    /**
     * {@inheritDoc}
     */
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getExternalUserEasyBeansInterceptors() {
        return ejbJarClassMetadata.getExternalUserEasyBeansInterceptors();
    }

    /**
     * {@inheritDoc}
     */
   public List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors() {
        return ejbJarClassMetadata.getGlobalEasyBeansInterceptors();
    }

   /**
    * {@inheritDoc}
    */
    public String[] getInterfaces() {
        return ejbJarClassMetadata.getInterfaces();
    }

    /**
     * {@inheritDoc}
     */
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getInternalUserEasyBeansInterceptors() {
        return ejbJarClassMetadata.getInternalUserEasyBeansInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public IJCommonBean getJCommonBean() {
        return ejbJarClassMetadata.getJCommonBean();
    }

    /**
     * {@inheritDoc}
     */
    public IJMessageDriven getJMessageDriven() {
        return ejbJarClassMetadata.getJMessageDriven();
    }

    /**
     * {@inheritDoc}
     */
    public IJStateful getJStateful() {
        return ejbJarClassMetadata.getJStateful();
    }

    /**
     * {@inheritDoc}
     */
    public IJStateless getJStateless() {
        return ejbJarClassMetadata.getJStateless();
    }

    /**
     * {@inheritDoc}
     */
    public String getLocalHome() {
        return ejbJarClassMetadata.getLocalHome();
    }

    /**
     * {@inheritDoc}
     */
    public IJLocal getLocalInterfaces() {
        return ejbJarClassMetadata.getLocalInterfaces();
    }

    /**
     * {@inheritDoc}
     */
    public String getRemoteHome() {
        return ejbJarClassMetadata.getRemoteHome();
    }

    /**
     * {@inheritDoc}
     */
    public String getRemoteInterface() {
        return ejbJarClassMetadata.getRemoteInterface();
    }

    /**
     * {@inheritDoc}
     */
    public IJRemote getRemoteInterfaces() {
        return ejbJarClassMetadata.getRemoteInterfaces();
    }

    /**
     * {@inheritDoc}
     */
    public List<String> getRolesAllowed() {
        return ejbJarClassMetadata.getRolesAllowed();
    }

    /**
     * {@inheritDoc}
     */
    public String getRunAs() {
        return ejbJarClassMetadata.getRunAs();
    }

    /**
     * {@inheritDoc}
     */
    public String getSuperName() {
        return ejbJarClassMetadata.getSuperName();
    }

    /**
     * {@inheritDoc}
     */
    public TransactionAttributeType getTransactionAttributeType() {
        return ejbJarClassMetadata.getTransactionAttributeType();
    }

    /**
     * {@inheritDoc}
     */
    public TransactionManagementType getTransactionManagementType() {
        return ejbJarClassMetadata.getTransactionManagementType();
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasPermitAll() {
        return ejbJarClassMetadata.hasPermitAll();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isBean() {
        return ejbJarClassMetadata.isBean();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isExcludedDefaultInterceptors() {
        return ejbJarClassMetadata.isExcludedDefaultInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isMdb() {
        return ejbJarClassMetadata.isMdb();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isOrderedInterceptors() {
        return ejbJarClassMetadata.isOrderedInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isSession() {
        return ejbJarClassMetadata.isSession();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isStateful() {
        return ejbJarClassMetadata.isStateful();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isStateless() {
        return ejbJarClassMetadata.isStateless();
    }

    /**
     * {@inheritDoc}
     */
    public void setAnnotationsInterceptors(final IJInterceptors annotationInterceptors) {
        ejbJarClassMetadata.setAnnotationsInterceptors(annotationInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setApplicationException(final ApplicationException applicationException) {
        ejbJarClassMetadata.setApplicationException(applicationException);
    }

    /**
     * {@inheritDoc}
     */
    public void setClassType(final ClassType type) {
        ejbJarClassMetadata.setClassType(type);
    }

    /**
     * {@inheritDoc}
     */
    public void setDeclareRoles(final List<String> declareRoles) {
        ejbJarClassMetadata.setDeclareRoles(declareRoles);
    }

    /**
     * {@inheritDoc}
     */
    public void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors) {
        ejbJarClassMetadata.setExcludeDefaultInterceptors(excludeDefaultInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setExternalUserInterceptors(
            final Map<InterceptorType, List<? extends IJClassInterceptor>> externalUserInterceptors) {
        ejbJarClassMetadata.setExternalUserInterceptors(externalUserInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors) {
        ejbJarClassMetadata.setGlobalEasyBeansInterceptors(globalEasyBeansInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setInterfaces(final String[] array) {
        ejbJarClassMetadata.setInterfaces(array);
    }

    /**
     * {@inheritDoc}
     */
    public void setJCommonBean(final IJCommonBean commonBean) {
        ejbJarClassMetadata.setJCommonBean(commonBean);
    }

    /**
     * {@inheritDoc}
     */
    public void setJMessageDriven(final IJMessageDriven messageDriven) {
        ejbJarClassMetadata.setJMessageDriven(messageDriven);
    }

    /**
     * {@inheritDoc}
     */
    public void setJStateful(final IJStateful stateful) {
        ejbJarClassMetadata.setJStateful(stateful);
    }

    /**
     * {@inheritDoc}
     */
    public void setJStateless(final IJStateless stateless) {
        ejbJarClassMetadata.setJStateless(stateless);
    }

    /**
     * {@inheritDoc}
     */
    public void setLocalHome(final String localHome) {
        ejbJarClassMetadata.setLocalHome(localHome);
    }

    /**
     * {@inheritDoc}
     */
    public void setLocalInterfaces(final IJLocal local) {
        ejbJarClassMetadata.setLocalInterfaces(local);
    }

    /**
     * {@inheritDoc}
     */
    public void setModified() {
        ejbJarClassMetadata.setModified();
    }

    /**
     * {@inheritDoc}
     */
    public void setOrderedInterceptors(final boolean orderedInterceptors) {
        ejbJarClassMetadata.setOrderedInterceptors(orderedInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setPermitAll(final boolean permitAll) {
        ejbJarClassMetadata.setPermitAll(permitAll);
    }

    /**
     * {@inheritDoc}
     */
    public void setRemoteHome(final String remoteHome) {
        ejbJarClassMetadata.setRemoteHome(remoteHome);
    }

    /**
     * {@inheritDoc}
     */
    public void setRemoteInterface(final String remoteInterface) {
        ejbJarClassMetadata.setRemoteInterface(remoteInterface);
    }

    /**
     * {@inheritDoc}
     */
    public void setRemoteInterfaces(final IJRemote remote) {
        ejbJarClassMetadata.setRemoteInterfaces(remote);
    }

    /**
     * {@inheritDoc}
     */
    public void setRolesAllowed(final List<String> rolesAllowed) {
        ejbJarClassMetadata.setRolesAllowed(rolesAllowed);
    }

    /**
     * {@inheritDoc}
     */
    public void setRunAs(final String runAs) {
        ejbJarClassMetadata.setRunAs(runAs);
    }

    /**
     * {@inheritDoc}
     */
    public void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType) {
        ejbJarClassMetadata.setTransactionAttributeType(transactionAttributeType);
    }

    /**
     * {@inheritDoc}
     */
    public void setTransactionManagementType(final TransactionManagementType transactionManagementType) {
        ejbJarClassMetadata.setTransactionManagementType(transactionManagementType);
    }

    /**
     * {@inheritDoc}
     */
    public boolean wasModified() {
        return ejbJarClassMetadata.wasModified();
    }

    /**
     * {@inheritDoc}
     */
    public void addStandardFieldMetadata(final IEjbJarFieldMetadata<E> fieldAnnotationMetadata) {
        ejbJarClassMetadata.addStandardFieldMetadata(fieldAnnotationMetadata);
    }

    /**
     * {@inheritDoc}
     */
    public void addStandardMethodMetadata(final IEjbJarMethodMetadata<E> methodAnnotationMetadata) {
        ejbJarClassMetadata.addStandardMethodMetadata(methodAnnotationMetadata);
    }

    /**
     * {@inheritDoc}
     */
    public IEjbJarFieldMetadata<E> getStandardFieldMetadata(final JField field) {
        return ejbJarClassMetadata.getStandardFieldMetadata(field);
    }

    /**
     * {@inheritDoc}
     */
    public Collection<? extends IEjbJarFieldMetadata<E>> getStandardFieldMetadataCollection() {
        return ejbJarClassMetadata.getStandardFieldMetadataCollection();
    }

    /**
     * {@inheritDoc}
     */
    public IEjbJarMethodMetadata<E> getStandardMethodMetadata(final JMethod method) {
        return ejbJarClassMetadata.getStandardMethodMetadata(method);
    }

    /**
     * {@inheritDoc}
     */
    public Collection<? extends IEjbJarMethodMetadata<E>> getStandardMethodMetadataCollection() {
        return ejbJarClassMetadata.getStandardMethodMetadataCollection();
    }

    /**
     * {@inheritDoc}
     */
    public List<? extends IEjbJarFieldMetadata<E>> searchStandardFieldMetadata(final String fieldName) {
        return ejbJarClassMetadata.searchStandardFieldMetadata(fieldName);
    }

    /**
     * {@inheritDoc}
     */
    public List<? extends IEjbJarMethodMetadata<E>> searchStandardMethodMetadata(final String methodName) {
        return ejbJarClassMetadata.searchStandardMethodMetadata(methodName);
    }

    /**
     * {@inheritDoc}
     */
    public Collection<IEnvEntry> getEnvEntryCollection() {
        return ejbJarClassMetadata.getEnvEntryCollection();
    }

    /**
     * {@inheritDoc}
     */
    public void setInternalUserInterceptors(
            final Map<InterceptorType, List<? extends IJClassInterceptor>> internalUserInterceptors) {
        ejbJarClassMetadata.setInternalUserInterceptors(internalUserInterceptors);
    }

    /**
     * @return the standard metadata
     */
    public IEjbJarClassMetadata<E> getStandard() {
        return ejbJarClassMetadata;
    }

}
