/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: SpecificEjbJarFieldMetadata.java 3345 2008-05-29 11:16:34Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.specific;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.scan.api.metadata.specific.ISpecificClassMetadata;
import org.ow2.util.scan.api.metadata.specific.ISpecificFieldMetadata;
import org.ow2.util.scan.api.metadata.specific.ISpecificMethodMetadata;
import org.ow2.util.scan.api.metadata.specific.SpecificFieldMetadata;
import org.ow2.util.scan.api.metadata.structures.JField;

/**
 * SpecificWarFieldMetadata.
 * @param <E> deployable type
 * @param <SC> specific class metadata
 * @param <SM> specific method metadata
 * @param <SF> specific field metadata
 * @author Gael Lalire
 */
public class SpecificEjbJarFieldMetadata<
    E extends EJBDeployable<E>,
    SC extends ISpecificClassMetadata<SC, SM, SF>,
    SM extends ISpecificMethodMetadata<SC, SM, SF>,
    SF extends ISpecificFieldMetadata<SC, SM, SF>>
        extends SpecificFieldMetadata<SC, SM, SF> implements IEjbJarFieldMetadata<E> {

    /**
     * The standard war method metadata.
     */
    private IEjbJarFieldMetadata<E> ejbJarFieldMetadata;

    /**
     * Construct delegation.
     * @param ejbJarFieldMetadata the standard ejb jar field metadata
     * @param classMetadata the specific class metadata (the parent)
     */
    public SpecificEjbJarFieldMetadata(final IEjbJarFieldMetadata<E> ejbJarFieldMetadata, final SC classMetadata) {
        super(classMetadata);
        this.ejbJarFieldMetadata = ejbJarFieldMetadata;
    }

    /**
     * {@inheritDoc}
     */
    public JField getJField() {
        return ejbJarFieldMetadata.getJField();
    }

    /**
     * {@inheritDoc}
     */
    public IJEjbEJB getJEjbEJB() {
        return ejbJarFieldMetadata.getJEjbEJB();
    }

    /**
     * {@inheritDoc}
     */
    public void setJEjbEJB(final IJEjbEJB ejbEJB) {
        ejbJarFieldMetadata.setJEjbEJB(ejbEJB);
    }

    /**
     * {@inheritDoc}
     */
    public IJAnnotationResource getJAnnotationResource() {
        return ejbJarFieldMetadata.getJAnnotationResource();
    }

    /**
     * {@inheritDoc}
     */
    public void setJAnnotationResource(final IJAnnotationResource annotationResource) {
        ejbJarFieldMetadata.setJAnnotationResource(annotationResource);
    }

    /**
     * {@inheritDoc}
     */
    public IJavaxPersistenceContext getJavaxPersistenceContext() {
        return ejbJarFieldMetadata.getJavaxPersistenceContext();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPersistenceContext() {
        return ejbJarFieldMetadata.isPersistenceContext();
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaxPersistenceContext(final IJavaxPersistenceContext javaxPersistenceContext) {
        ejbJarFieldMetadata.setJavaxPersistenceContext(javaxPersistenceContext);
    }

    /**
     * {@inheritDoc}
     */
    public IJavaxPersistenceUnit getJavaxPersistenceUnit() {
        return ejbJarFieldMetadata.getJavaxPersistenceUnit();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPersistenceUnit() {
        return ejbJarFieldMetadata.isPersistenceUnit();
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaxPersistenceUnit(final IJavaxPersistenceUnit javaxPersistenceUnit) {
        ejbJarFieldMetadata.setJavaxPersistenceUnit(javaxPersistenceUnit);
    }

    /**
     * {@inheritDoc}
     */
    public String getFieldName() {
        return ejbJarFieldMetadata.getFieldName();
    }

    /**
     * {@inheritDoc}
     */
    public IEjbJarFieldMetadata<E> attachClone(final IEjbJarClassMetadata<E> classMetadata) throws CloneNotSupportedException {
        return ejbJarFieldMetadata.attachClone(classMetadata);
    }

    /**
     * {@inheritDoc}
     */
    public IEjbJarClassMetadata<E> getClassMetadata() {
        return ejbJarFieldMetadata.getClassMetadata();
    }

    /**
     * @return the standard metadata
     */
    public IEjbJarFieldMetadata<E> getStandard() {
        return ejbJarFieldMetadata;
    }


}
