/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: SpecificEjbJarMethodMetadata.java 3340 2008-05-29 10:12:57Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.specific;

import java.util.List;
import java.util.Map;

import javax.ejb.Remove;
import javax.ejb.TransactionAttributeType;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.TransactionAttributeLevel;
import org.ow2.util.scan.api.metadata.specific.ISpecificClassMetadata;
import org.ow2.util.scan.api.metadata.specific.ISpecificFieldMetadata;
import org.ow2.util.scan.api.metadata.specific.ISpecificMethodMetadata;
import org.ow2.util.scan.api.metadata.specific.SpecificMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * SpecificWarMethodMetadata.
 * @param <E> deployable type
 * @param <SC> specific class metadata
 * @param <SM> specific method metadata
 * @param <SF> specific field metadata
 * @author Gael Lalire
 */
public class SpecificEjbJarMethodMetadata<
    E extends EJBDeployable<E>,
    SC extends ISpecificClassMetadata<SC, SM, SF>,
    SM extends ISpecificMethodMetadata<SC, SM, SF>,
    SF extends ISpecificFieldMetadata<SC, SM, SF>>
        extends SpecificMethodMetadata<SC, SM, SF> implements IEjbJarMethodMetadata<E> {

    /**
     * The standard war method metadata.
     */
    private IEjbJarMethodMetadata<E> ejbJarMethodMetadata;

    /**
     * Construct delegation.
     * @param ejbJarMethodMetadata the standard ejb jar method metadata
     * @param classMetadata the specific class metadata (the parent)
     */
    protected SpecificEjbJarMethodMetadata(final IEjbJarMethodMetadata<E> ejbJarMethodMetadata, final SC classMetadata) {
        super(classMetadata);
        this.ejbJarMethodMetadata = ejbJarMethodMetadata;
    }

    /**
     * {@inheritDoc}
     */
    public JMethod getJMethod() {
        return ejbJarMethodMetadata.getJMethod();
    }

    /**
     * {@inheritDoc}
     */
    public IJEjbEJB getJEjbEJB() {
        return ejbJarMethodMetadata.getJEjbEJB();
    }

    /**
     * {@inheritDoc}
     */
    public void setJEjbEJB(final IJEjbEJB ejbEJB) {
        ejbJarMethodMetadata.setJEjbEJB(ejbEJB);
    }

    /**
     * {@inheritDoc}
     */
    public IJAnnotationResource getJAnnotationResource() {
        return ejbJarMethodMetadata.getJAnnotationResource();
    }

    /**
     * {@inheritDoc}
     */
    public void setJAnnotationResource(final IJAnnotationResource annotationResource) {
        ejbJarMethodMetadata.setJAnnotationResource(annotationResource);
    }

    /**
     * {@inheritDoc}
     */
    public IJavaxPersistenceContext getJavaxPersistenceContext() {
        return ejbJarMethodMetadata.getJavaxPersistenceContext();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPersistenceContext() {
        return ejbJarMethodMetadata.isPersistenceContext();
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaxPersistenceContext(final IJavaxPersistenceContext javaxPersistenceContext) {
        ejbJarMethodMetadata.setJavaxPersistenceContext(javaxPersistenceContext);
    }

    /**
     * {@inheritDoc}
     */
    public IJavaxPersistenceUnit getJavaxPersistenceUnit() {
        return ejbJarMethodMetadata.getJavaxPersistenceUnit();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPersistenceUnit() {
        return ejbJarMethodMetadata.isPersistenceUnit();
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaxPersistenceUnit(final IJavaxPersistenceUnit javaxPersistenceUnit) {
        ejbJarMethodMetadata.setJavaxPersistenceUnit(javaxPersistenceUnit);
    }

    /**
     * {@inheritDoc}
     */
    public void setJMethod(final JMethod method) {
        ejbJarMethodMetadata.setJMethod(method);
    }

    /**
     * {@inheritDoc}
     */
    public IJInterceptors getAnnotationInterceptors() {
        return ejbJarMethodMetadata.getAnnotationInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors() {
        return ejbJarMethodMetadata.getGlobalEasyBeansInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public List<? extends IJClassInterceptor> getInterceptors() {
        return ejbJarMethodMetadata.getInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public Remove getJRemove() {
        return ejbJarMethodMetadata.getJRemove();
    }

    /**
     * {@inheritDoc}
     */
    public String getMethodName() {
        return ejbJarMethodMetadata.getMethodName();
    }

    /**
     * {@inheritDoc}
     */
    public List<String> getRolesAllowed() {
        return ejbJarMethodMetadata.getRolesAllowed();
    }

    /**
     * {@inheritDoc}
     */
    public TransactionAttributeLevel getTransactionAttributeLevel() {
        return ejbJarMethodMetadata.getTransactionAttributeLevel();
    }

    /**
     * {@inheritDoc}
     */
    public TransactionAttributeType getTransactionAttributeType() {
        return ejbJarMethodMetadata.getTransactionAttributeType();
    }

    /**
     * {@inheritDoc}
     */
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getUserEasyBeansInterceptors() {
        return ejbJarMethodMetadata.getUserEasyBeansInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasDenyAll() {
        return ejbJarMethodMetadata.hasDenyAll();
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasPermitAll() {
        return ejbJarMethodMetadata.hasPermitAll();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAroundInvoke() {
        return ejbJarMethodMetadata.isAroundInvoke();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isBusinessMethod() {
        return ejbJarMethodMetadata.isBusinessMethod();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isExcludedClassInterceptors() {
        return ejbJarMethodMetadata.isExcludedClassInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isExcludedDefaultInterceptors() {
        return ejbJarMethodMetadata.isExcludedDefaultInterceptors();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isLifeCycleMethod() {
        return ejbJarMethodMetadata.isLifeCycleMethod();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPostActivate() {
        return ejbJarMethodMetadata.isPostActivate();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPostConstruct() {
        return ejbJarMethodMetadata.isPostConstruct();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPreDestroy() {
        return ejbJarMethodMetadata.isPreDestroy();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isPrePassivate() {
        return ejbJarMethodMetadata.isPrePassivate();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isTimeout() {
        return ejbJarMethodMetadata.isTimeout();
    }

    /**
     * {@inheritDoc}
     */
    public void setAnnotationsInterceptors(final IJInterceptors annotationInterceptors) {
        ejbJarMethodMetadata.setAnnotationsInterceptors(annotationInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setAroundInvoke(final boolean aroundInvoke) {
        ejbJarMethodMetadata.setAroundInvoke(aroundInvoke);
    }

    /**
     * {@inheritDoc}
     */
    public void setBusinessMethod(final boolean flag) {
        ejbJarMethodMetadata.setBusinessMethod(flag);
    }

    /**
     * {@inheritDoc}
     */
    public void setDenyAll(final boolean denyAll) {
        ejbJarMethodMetadata.setDenyAll(denyAll);
    }

    /**
     * {@inheritDoc}
     */
    public void setExcludeClassInterceptors(final boolean excludeClassInterceptors) {
        ejbJarMethodMetadata.setExcludeClassInterceptors(excludeClassInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors) {
        ejbJarMethodMetadata.setExcludeDefaultInterceptors(excludeDefaultInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors) {
        ejbJarMethodMetadata.setGlobalEasyBeansInterceptors(globalEasyBeansInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setInterceptors(final List<? extends IJClassInterceptor> interceptors) {
        ejbJarMethodMetadata.setInterceptors(interceptors);
    }

    /**
     * {@inheritDoc}
     */
    public void setPermitAll(final boolean permitAll) {
        ejbJarMethodMetadata.setPermitAll(permitAll);
    }

    /**
     * {@inheritDoc}
     */
    public void setPostActivate(final boolean postActivate) {
        ejbJarMethodMetadata.setPostActivate(postActivate);
    }

    /**
     * {@inheritDoc}
     */
    public void setPostConstruct(final boolean postConstruct) {
        ejbJarMethodMetadata.setPostConstruct(postConstruct);
    }

    /**
     * {@inheritDoc}
     */
    public void setPreDestroy(final boolean preDestroy) {
        ejbJarMethodMetadata.setPreDestroy(preDestroy);
    }

    /**
     * {@inheritDoc}
     */
    public void setPrePassivate(final boolean prePassivate) {
        ejbJarMethodMetadata.setPrePassivate(prePassivate);
    }

    /**
     * {@inheritDoc}
     */
    public void setRemove(final Remove remove) {
        ejbJarMethodMetadata.setRemove(remove);
    }

    /**
     * {@inheritDoc}
     */
    public void setRolesAllowed(final List<String> rolesAllowed) {
        ejbJarMethodMetadata.setRolesAllowed(rolesAllowed);
    }

    /**
     * {@inheritDoc}
     */
    public void setTimeout(final boolean timeout) {
        ejbJarMethodMetadata.setTimeout(timeout);
    }

    /**
     * {@inheritDoc}
     */
    public void setTransactionAttributeLevel(final TransactionAttributeLevel transactionAttributeLevel) {
        ejbJarMethodMetadata.setTransactionAttributeLevel(transactionAttributeLevel);
    }

    /**
     * {@inheritDoc}
     */
    public void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType) {
        ejbJarMethodMetadata.setTransactionAttributeType(transactionAttributeType);
    }

    /**
     * {@inheritDoc}
     */
    public void setUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> userInterceptors) {
        ejbJarMethodMetadata.setUserInterceptors(userInterceptors);
    }

    /**
     * {@inheritDoc}
     */
    public IEjbJarMethodMetadata<E> attachClone(final IEjbJarClassMetadata<E> classMetadata) throws CloneNotSupportedException {
        return ejbJarMethodMetadata.attachClone(classMetadata);
    }

    /**
     * {@inheritDoc}
     */
    public IEjbJarClassMetadata<E> getClassMetadata() {
        return ejbJarMethodMetadata.getClassMetadata();
    }

    /**
     * {@inheritDoc}
     */
    public void setClassMetadata(final IEjbJarClassMetadata<E> classMetadata) {
        ejbJarMethodMetadata.setClassMetadata(classMetadata);
    }

    /**
     * @return the standard metadata
     */
    public IEjbJarMethodMetadata<E> getStandard() {
        return ejbJarMethodMetadata;
    }

    @Override
    public Object clone() {
        try {
            SpecificEjbJarMethodMetadata<E, SC, SM, SF> clone = (SpecificEjbJarMethodMetadata<E, SC, SM, SF>) super.clone();
            clone.ejbJarMethodMetadata = (IEjbJarMethodMetadata<E>) ejbJarMethodMetadata.clone();
            return clone;
        } catch (CloneNotSupportedException e) {
            return null;
        }
    }

}
