/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JCommonBean.java 2950 2008-04-25 09:14:23Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.struct;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;

/**
 * Defines common methods used by Session Bean and Message Driven beans.
 * @author Florent Benoit
 */
public class JCommonBean implements IJCommonBean {

    /**
     * Name of the bean.
     */
    private String name = null;

    /**
     * Default name ? (annotation)
     */
    private boolean defaultName = true;

    /**
     * Mapped name (could be used as JNDI name).
     */
    private String mappedName = null;

    /**
     * Description.
     */
    private String description = null;

    /**
     * List of alias (for the different ejb-name).
     * FIXME: maybe chaneg this to a different way
     */
    private List<String> aliases = null;

    /**
     * Build an object that will be shared by EJB (Session + MDB).
     */
    public JCommonBean() {

    }

    /**
     * @return the description.
     */
    public String getDescription() {
        return description;
    }

    /**
     * Sets the description.
     * @param description value of description
     */
    public void setDescription(final String description) {
        this.description = description;
    }

    /**
     * @return the mapped name (JNDI ?)
     */
    public String getMappedName() {
        return mappedName;
    }

    /**
     * Sets the mapped name.
     * @param mappedName the value to set
     */
    public void setMappedName(final String mappedName) {
        this.mappedName = mappedName;
    }

    /**
     * @return name of the bean.
     */
    public String getName() {
        return name;
    }

    /**
     * Sets the bean name.
     * @param name the bean's name
     */
    public void setName(final String name) {
        this.name = name;
    }

    /**
     * @return true if the bean is the default name (set with annotation)
     */
    public boolean isDefaultName() {
        return defaultName;
    }

    /**
     * Is that this name is a default name ?
     * @param defaultName boolean true/false
     */
    public void setDefaultName(final boolean defaultName) {
        this.defaultName = defaultName;
    }

    /**
     * @return a list of aliases (ejb-names)
     */
    public List<String> getAliases() {
        return aliases;
    }

    /**
     * Adds an alias to the list of the names.
     * @param ejbName the name of the ejb
     */
    public void addAlias(final String ejbName) {
        if (aliases == null) {
            aliases = new ArrayList<String>();
        }
        aliases.add(ejbName);
    }


    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));

        // defaultName
        sb.append("[isDefaultName=");
        sb.append(defaultName);

        // name
        sb.append(",name=");
        sb.append(name);

        // mappedName
        sb.append(",mappedName=");
        sb.append(mappedName);

        // description
        sb.append(",description=");
        sb.append(description);

        // ejb-name aliases
        if (aliases != null && aliases.size() > 0) {
            sb.append(",aliases=");
            sb.append(aliases);
        }

        sb.append("]");
        return sb.toString();
    }
}
