/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JInterface.java 2950 2008-04-25 09:14:23Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.struct;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.ejbjar.api.struct.IJInterface;

/**
 * List of interfaces.
 * @author Florent Benoit
 */
public abstract class JInterface implements IJInterface {

    /**
     * List of interfaces.
     */
    private List<String> interfaces = null;

    /**
     * Constructor.
     */
    public JInterface() {
        interfaces = new ArrayList<String>();
    }

    /**
     * Add an interface.
     * @param itf name of the interface (asm)
     */
    public void addInterface(final String itf) {
        interfaces.add(itf);
    }

    /**
     * @return list of interfaces.
     */
    public List<String> getInterfaces() {
        return interfaces;
    }

    /**
     * @param itf the name of an encoded interface.
     * @return true if the interface is already in the list.
     */
    public boolean contains(final String itf) {
        return interfaces.contains(itf);
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));

        // list
        sb.append(interfaces);
        return sb.toString();
    }
}
