/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JMessageDriven.java 2950 2008-04-25 09:14:23Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.struct;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.ActivationConfigProperty;

import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;

/**
 * Acts as an implementation of &#64;{@link javax.ejb.MessageDriven} annotation.
 * @author Florent Benoit
 */
public class JMessageDriven extends JCommonBean implements IJMessageDriven {

    /**
     * List of ActivationConfigProperty.
     */
    private List<ActivationConfigProperty> activationConfigProperties = null;

    /**
     * Message listener Interface.
     */
    private String messageListenerInterface = null;

    /**
     * Build an object which represents &#64;{@link javax.ejb.MessageDriven} object.
     */
    public JMessageDriven() {
        super();
        activationConfigProperties = new ArrayList<ActivationConfigProperty>();
    }

    /**
     * Adds an activation config property.
     * @param activationConfigProperty object to add in the list
     */
    public void addActivationConfigProperty(final ActivationConfigProperty activationConfigProperty) {
        activationConfigProperties.add(activationConfigProperty);
    }

    /**
     * Gets the activation config properties.
     * @return the list of activation config properties
     */
    public List<ActivationConfigProperty> getActivationConfigProperties() {
        return activationConfigProperties;
    }

    /**
     * @return message listener interface.
     */
    public String getMessageListenerInterface() {
        return messageListenerInterface;
    }

    /**
     * Sets the message listener interface.
     * @param messageListenerInterface the given interface.
     */
    public void setMessageListenerInterface(final String messageListenerInterface) {
        this.messageListenerInterface = messageListenerInterface;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));

        sb.append(super.toString());

        // messageListenerInterface
        sb.append("[messageListenerInterface=");
        sb.append(messageListenerInterface);

        // property value
        sb.append(", activationConfigProperties=");
        sb.append(activationConfigProperties);

        sb.append("]");
        return sb.toString();
    }
}
