/**
 * EasyBeans
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CommonMetadataMerge.java 3319 2008-05-27 13:00:31Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.helper;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarDeployableMetadata;

/**
 * Defines common stuff that can be used by merger of Deployment Descriptor into
 * the metadata.
 * @param <E> the deployable type
 * @author Florent BENOIT
 */
public class CommonMetadataMerge<E extends EJBDeployable<E>> {

    /**
     * Metadata for an ejb-jar.
     */
    private EjbJarDeployableMetadata<E> ejbMetadata = null;

    /**
     * Logger.
     */
    //private static Log logger = LogFactory.getLog(CommonMetadataMerge.class);

    /**
     * Constructor that can be used by subclass.
     * @param ejbMetadata the metadata corresponding to an EJB-JAR file.
     */
    protected CommonMetadataMerge(final EjbJarDeployableMetadata<E> ejbMetadata) {
        this.ejbMetadata = ejbMetadata;
    }

    /**
     * @return the EJB metadata of this merger.
     */
    public EjbJarDeployableMetadata<E> getEjbJarDeployableMetadata() {
        return ejbMetadata;
    }

    /**
     * Find/Gets a classAnnotationMetadata for a given ejb name and/or a given
     * ejbclass name.
     * @param ejbName the name of the EJB.
     * @param ejbClass the class of the EJB.
     * @return a classAnnotationMetadata if found, else exception
     */
    protected IEjbJarClassMetadata<E> findClassForEjb(final String ejbName, final String ejbClass) {
        IEjbJarClassMetadata<E> classAnnotationMetadata = null;

        // First, try with ejbClass if not null
        if (ejbClass != null) {
            classAnnotationMetadata = ejbMetadata.getEjbJarClassMetadata(encode(ejbClass));
            // found ? return it
            if (classAnnotationMetadata != null) {
                return classAnnotationMetadata;
            }
        }

        // else, needs to find a class with the specified name
        for (IEjbJarClassMetadata<E> tmpMetaData : ejbMetadata.getEjbJarClassMetadataCollection()) {
            if (tmpMetaData.isBean()) {
                // check name
                IJCommonBean bean = tmpMetaData.getJCommonBean();
                if (bean != null) {
                    // found !
                    if (ejbName != null && ejbName.equals(bean.getName())) {
                        return tmpMetaData;
                    }
                }
            }
        }
        throw new IllegalStateException("No class with ejb-name '" + ejbName + "' or ejb-class '" + ejbClass
                + "' was found in the EJB-JAR file '" + ejbMetadata);
    }

    /**
     * Encode a classname (replace . by /) as it's the internal form of the
     * metadata
     * @param className the name of the class to encode
     * @return encoded name
     */
    protected static String encode(final String className) {
        return className.replace(".", "/");
    }

}
