package org.ow2.util.ee.metadata.ejbjar.impl.xml.helper;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EJB3;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EnterpriseBeans;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Interceptor;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Interceptors;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MessageDriven;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Session;

/**
 * This utility class extract data from xml descriptor.
 * @author Gael Lalire
 */
public final class ExtractUsefullClasses {

    /**
     * Utility class.
     */
    private ExtractUsefullClasses() {
    }

    /**
     * @param ejb3 the deployment descriptor object
     * @return list of needed classes
     */
    public static List<String> extract(final EJB3 ejb3) {
        List<String> classNameList = new ArrayList<String>();
        if (ejb3 != null) {
            // Get enterprise beans classes
            EnterpriseBeans enterpriseBeans = ejb3.getEnterpriseBeans();
            if (enterpriseBeans != null) {
                // get session beans
                List<Session> sessionList = enterpriseBeans.getSessionList();
                for (Session session : sessionList) {
                    // get ejb-class
                    classNameList.add(session.getEjbClass());
                }
                List<MessageDriven> messageDrivenList = enterpriseBeans.getMessageDrivenList();
                for (MessageDriven messageDriven : messageDrivenList) {
                    // get ejb-class
                    classNameList.add(messageDriven.getEjbClass());
                }
            }
            // Get interceptor classes
            Interceptors interceptors = ejb3.getInterceptors();
            if (interceptors != null) {
                List<Interceptor> interceptorList = interceptors.getInterceptorList();
                if (interceptorList != null) {
                    for (Interceptor interceptor : interceptorList) {
                        classNameList.add(interceptor.getInterceptorClass());
                    }
                }
            }

        }
        return classNameList;
    }

}
