/**
 * EasyBeans
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MetadataMerge.java 3524 2008-06-20 13:37:06Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.helper;

import static org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.LifeCycleCallback.POST_ACTIVATE;
import static org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.LifeCycleCallback.POST_CONSTRUCT;
import static org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.LifeCycleCallback.PRE_DESTROY;
import static org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.LifeCycleCallback.PRE_PASSIVATE;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagementType;

import org.ow2.util.asm.Type;
import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.interfaces.ISharedMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.common.impl.struct.JAnnotationResource;
import org.ow2.util.ee.metadata.common.impl.struct.JEjbEJB;
import org.ow2.util.ee.metadata.common.impl.struct.JInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.ClassType;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.TransactionAttributeLevel;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJLocal;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJRemote;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JActivationConfigProperty;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JApplicationException;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JCommonBean;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JLocal;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JRemote;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JRemove;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.AbsBean;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.AssemblyDescriptor;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.ContainerTransaction;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EJB3;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EnterpriseBeans;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Interceptor;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.InterceptorBinding;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Interceptors;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MessageDriven;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MethodPermission;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.RemoveMethod;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Session;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.AbsEJBRef;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.AbsEnvironment;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.ActivationConfig;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.AroundInvoke;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.EJBLocalRef;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.EJBRef;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.EnvEntry;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.InjectionTarget;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.LifeCycleCallback;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.MessageDestinationRef;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.MethodDD;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.ResourceEnvRef;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.ResourceRef;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class merge the information of the struct representing EJB DD into the
 * metadata.
 * @param <E> deployable type
 * @author Florent Benoit
 */
public final class MetadataMerge<E extends EJBDeployable<E>> extends CommonMetadataMerge<E> {

    /**
     * Wildcard attribute.
     */
    private static final String WILDCARD = "*";

    /**
     * Defines java.lang.Object class.
     */
    public static final String JAVA_LANG_OBJECT = "java/lang/Object";

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(MetadataMerge.class);

    /**
     * Helper class, no public constructor.
     * @param ejbMetadata the metadata corresponding to an EJB-JAR file.
     */
    private MetadataMerge(final EjbJarDeployableMetadata<E> ejbMetadata) {
        super(ejbMetadata);
    }

    /**
     * Takes struct of metadata and adds/set information on class/methods
     * metadata.
     * @param ejbMetadata the metadata corresponding to an EJB-JAR file.
     * @param <T> deployable type
     */
    public static <T extends EJBDeployable<T>> void merge(final EjbJarDeployableMetadata<T> ejbMetadata) {
        new MetadataMerge<T>(ejbMetadata).resolve();
    }

    /**
     * Do all merging operations.
     */
    private void resolve() {

        EJB3 ejb3 = getEjbJarDeployableMetadata().getEjb3();
        if (ejb3 != null) {
            // Get enterprise beans
            EnterpriseBeans enterpriseBeans = ejb3.getEnterpriseBeans();
            if (enterpriseBeans != null) {

                // get session beans
                List<Session> sessionList = enterpriseBeans.getSessionList();
                for (Session session : sessionList) {
                    // get ejb-name
                    String ejbName = session.getEjbName();
                    String ejbClass = session.getEjbClass();
                    IEjbJarClassMetadata<E> classAnnotationMetadata = findClassForEjb(ejbName, ejbClass);

                    // class is found, then apply all customization.
                    applySessionBean(session, classAnnotationMetadata);
                }

                // MDB
                List<MessageDriven> messageDrivenList = enterpriseBeans.getMessageDrivenList();
                for (MessageDriven messageDriven : messageDrivenList) {
                    // get ejb-name
                    String ejbName = messageDriven.getEjbName();
                    String ejbClass = messageDriven.getEjbClass();
                    IEjbJarClassMetadata<E> classAnnotationMetadata = findClassForEjb(ejbName, ejbClass);

                    // class is found, then apply all customization.
                    applyMessageDrivenBean(messageDriven, classAnnotationMetadata);
                }

            }

            // Interceptors
            Interceptors interceptors = ejb3.getInterceptors();
            if (interceptors != null) {
                applyInterceptors(interceptors);
            }

            // Assembly descriptor
            AssemblyDescriptor assemblyDescriptor = ejb3.getAssemblyDescriptor();
            if (assemblyDescriptor != null) {

                // merge application-exception
                mergeApplicationException(assemblyDescriptor);

                // merge interceptor-binding
                mergeInterceptorBinding(assemblyDescriptor);

                // merge container-transaction
                mergeContainerTransaction(assemblyDescriptor);

                // merge security
                mergeAssemblySecurity(assemblyDescriptor);

            }
        }
    }

    /**
     * Apply rules for the interceptors defined in the XML DD.
     * @param interceptors the interceptors struct representing
     *        &lt;interceptors&gt; element
     */
    private void applyInterceptors(final Interceptors interceptors) {
        // Get interceptors
        List<Interceptor> interceptorList = interceptors.getInterceptorList();
        if (interceptorList != null) {
            for (Interceptor interceptor : interceptorList) {

                // Check that there is an interceptor class defined
                String itCptClass = interceptor.getInterceptorClass();
                if (itCptClass == null) {
                    throw new IllegalStateException("Invalid interceptor with name '" + interceptor + "': No interceptor-class");
                }

                // Class Metadata
                IEjbJarClassMetadata<E> itcpMetadata = getEjbJarDeployableMetadata().getEjbJarClassMetadata(encode(itCptClass));
                if (itcpMetadata == null) {
                    throw new IllegalStateException("No metadata found for class '" + itCptClass + "'.");
                }

                // Will use metadata of the interceptor class
                applyJndiEnvironmentRefsGroup(interceptor, itcpMetadata);

            }
        }
    }

    /**
     * Apply rules for a session bean.
     * @param sessionbean the struct (XML DD)
     * @param classAnnotationMetadata (Annotation metadata)
     */
    private void applySessionBean(final Session sessionbean, final IEjbJarClassMetadata<E> classAnnotationMetadata) {
        applyCommonBean(sessionbean, classAnnotationMetadata);

        // set session-type
        String sessionType = sessionbean.getSessionType();
        if (sessionType == null && !classAnnotationMetadata.isSession()) {
            throw new IllegalStateException("Missing session-type for bean '" + sessionbean + "' and no annotation in '"
                    + classAnnotationMetadata + "'.");
        }
        // set the type of bean
        if ("Stateless".equals(sessionType)) {
            classAnnotationMetadata.setClassType(ClassType.STATELESS);
        } else if ("Stateful".equals(sessionType)) {
            classAnnotationMetadata.setClassType(ClassType.STATEFUL);
        }

        // Remote Home
        String remoteHome = sessionbean.getRemoteHome();
        if (remoteHome != null) {
            classAnnotationMetadata.setRemoteHome(remoteHome);
        }

        // Local Home
        String localHome = sessionbean.getLocalHome();
        if (localHome != null) {
            classAnnotationMetadata.setLocalHome(localHome);
        }

        // local interface
        if (sessionbean.getBusinessLocalList().size() > 0) {
            IJLocal jLocal = classAnnotationMetadata.getLocalInterfaces();
            if (jLocal == null) {
                jLocal = new JLocal();
                classAnnotationMetadata.setLocalInterfaces(jLocal);
            }
            for (String itf : sessionbean.getBusinessLocalList()) {
                String encodedItf = encode(itf);
                if (!jLocal.contains(encodedItf)) {
                    jLocal.addInterface(encodedItf);
                }
            }
        }

        // Mapped name
        if (sessionbean.getMappedName() != null) {
            IJCommonBean jCommonBean = classAnnotationMetadata.getJCommonBean();
            if (jCommonBean == null) {
                throw new IllegalStateException("No JCommonBean attribute on class '" + classAnnotationMetadata.getClassName()
                        + "' while there is a mapped name.");
            }
            jCommonBean.setMappedName(sessionbean.getMappedName());
        }

        // remote interface
        if (sessionbean.getBusinessRemoteList().size() > 0) {
            IJRemote jRemote = classAnnotationMetadata.getRemoteInterfaces();
            if (jRemote == null) {
                jRemote = new JRemote();
                classAnnotationMetadata.setRemoteInterfaces(jRemote);
            }
            for (String itf : sessionbean.getBusinessRemoteList()) {
                String encodedItf = encode(itf);
                if (!jRemote.contains(encodedItf)) {
                    jRemote.addInterface(encodedItf);
                }
            }
        }

        // Remove methods (for stateful bean)
        if (sessionbean.getRemoveMethodList().size() > 0) {
            if (!classAnnotationMetadata.isStateful()) {
                throw new IllegalStateException(
                        "Cannot apply XML DD remove methods on a bean which is not a stateful bean. Bean class is '"
                                + classAnnotationMetadata.getClassName() + "'. Remove Methods are '"
                                + sessionbean.getRemoveMethodList() + "'.");
            }

            for (RemoveMethod removeMethod : sessionbean.getRemoveMethodList()) {
                MethodDD method = removeMethod.getMethod();

                // method case
                List<? extends IEjbJarMethodMetadata<?>> methods = getMethodsForGivenMethodDD(method, classAnnotationMetadata);
                // No method found (warning)
                if (methods.size() == 0) {
                    logger.warn("No method found for the remove-method ''{0}'' on the class ''{1}''", method,
                            classAnnotationMetadata.getClassName());
                } else if (methods.size() > 1) {
                    logger.warn("Found more than 1 method for the remove-method ''{0}'' on the class ''{1}''."
                            + "All these methods will be set as remove methods.", method, classAnnotationMetadata
                            .getClassName());
                }

                // Set as a remove method
                for (IEjbJarMethodMetadata<?> methodAnnotationMetadata : methods) {

                    // Get existing value
                    Remove previousRemove = methodAnnotationMetadata.getJRemove();

                    // Value of the XML (for retain)
                    Boolean retainXML = removeMethod.getRetainIfException();

                    // If value has been defined in the XML, use this value,
                    // else use the value of the annotation, else the default
                    // value (false)
                    boolean retain = false;
                    if (retainXML != null) {
                        retain = retainXML.booleanValue();
                    } else if (previousRemove != null) {
                        // got value of the annotation
                        retain = previousRemove.retainIfException();
                    }

                    // Set the new value
                    JRemove jRemove = new JRemove(retain);
                    methodAnnotationMetadata.setRemove(jRemove);
                }
            }
        }

        // add env entries to metadata
        for (EnvEntry envEntry : sessionbean.getEnvEntryList()) {
            classAnnotationMetadata.getEnvEntryCollection().add(
                    new org.ow2.util.ee.metadata.ejbjar.impl.struct.EnvEntry(envEntry.getEnvEntryName(), envEntry
                            .getEnvEntryType(), envEntry.getEnvEntryValue()));
        }
    }

    /**
     * Apply rules for a Message Driven Bean.
     * @param messageDrivenBean the struct (XML DD)
     * @param classAnnotationMetadata (Annotation metadata)
     */
    private void applyMessageDrivenBean(final MessageDriven messageDrivenBean,
            final IEjbJarClassMetadata<E> classAnnotationMetadata) {
        applyCommonBean(messageDrivenBean, classAnnotationMetadata);

        // TODO: messaging-type
        String messagingType = messageDrivenBean.getMessagingType();

        // Set MDB type
        classAnnotationMetadata.setClassType(ClassType.MDB);

        // Activation config
        List<ActivationConfig> activationConfigList = messageDrivenBean.getActivationConfigList();
        for (ActivationConfig activationConfig : activationConfigList) {
            IJMessageDriven jMessageDriven = classAnnotationMetadata.getJMessageDriven();
            if (jMessageDriven == null) {
                jMessageDriven = new JMessageDriven();
                classAnnotationMetadata.setJMessageDriven(jMessageDriven);
            }

            JActivationConfigProperty actProperty = new JActivationConfigProperty(activationConfig.getName(), activationConfig
                    .getValue());
            jMessageDriven.addActivationConfigProperty(actProperty);
        }

    }

    /**
     * Apply rules for a common bean.
     * @param bean the struct (XML DD)
     * @param classAnnotationMetadata (Annotation metadata)
     */
    private void applyCommonBean(final AbsBean bean, final IEjbJarClassMetadata<E> classAnnotationMetadata) {
        // no Common bean, create one.
        if (classAnnotationMetadata.getJCommonBean() == null) {
            classAnnotationMetadata.setJCommonBean(new JCommonBean());
        }
        // Sets the ejb-name
        IJCommonBean jCommonBean = classAnnotationMetadata.getJCommonBean();
        boolean isDefaultName = jCommonBean.isDefaultName();
        if (isDefaultName) {
            jCommonBean.setName(bean.getEjbName());
            jCommonBean.setDefaultName(false);
        } else {
            // Bean already registered with this name, add an alias
            jCommonBean.addAlias(bean.getEjbName());
        }

        // Mapped name
        if (bean.getMappedName() != null) {
            if (jCommonBean == null) {
                throw new IllegalStateException("No JCommonBean attribute on class '" + classAnnotationMetadata.getClassName()
                        + "' while there is a mapped name.");
            }
            jCommonBean.setMappedName(bean.getMappedName());
        }

        // Apply run-as role
        String runAsRole = bean.getRunAsRole();
        if (runAsRole != null) {
            classAnnotationMetadata.setRunAs(runAsRole);
        }

        // Transaction type
        String transactionType = bean.getTransactionType();
        if (transactionType != null) {
            if ("Bean".equals(transactionType)) {
                classAnnotationMetadata.setTransactionManagementType(TransactionManagementType.BEAN);
            } else if ("Container".equals(transactionType)) {
                classAnnotationMetadata.setTransactionManagementType(TransactionManagementType.CONTAINER);
            } else {
                throw new IllegalStateException("Invalid transaction type '" + transactionType + "' found");
            }
        }

        applyJndiEnvironmentRefsGroup(bean, classAnnotationMetadata);

    }

    /**
     * Apply rules for a an environment.
     * @param envGroup the struct (XML DD)
     * @param classAnnotationMetadata (Annotation metadata)
     */
    private void applyJndiEnvironmentRefsGroup(final AbsEnvironment envGroup,
            final IEjbJarClassMetadata<E> classAnnotationMetadata) {

        // post-construct / pre-destroy
        applyLifeCycle(envGroup.getPostConstructCallbackList(), classAnnotationMetadata, POST_CONSTRUCT);
        applyLifeCycle(envGroup.getPreDestroyCallbackList(), classAnnotationMetadata, PRE_DESTROY);
        applyLifeCycle(envGroup.getPrePassivateCallbackList(), classAnnotationMetadata, PRE_PASSIVATE);
        applyLifeCycle(envGroup.getPostActivateCallbackList(), classAnnotationMetadata, POST_ACTIVATE);
        applyAroundInvoke(envGroup.getAroundInvokeList(), classAnnotationMetadata, AroundInvoke.NAME);

        // ejb(local) ref
        applyEjbRef(envGroup.getEJBRefList(), classAnnotationMetadata);
        applyEjbLocalRef(envGroup.getEJBLocalRefList(), classAnnotationMetadata);

        // resource(env) ref
        applyResourceRef(envGroup.getResourceRefList(), classAnnotationMetadata);
        applyResourceEnvRef(envGroup.getResourceEnvRefList(), classAnnotationMetadata);

        // message-destination-ref
        applyMessageDestinationRef(envGroup.getMessageDestinationRefList(), classAnnotationMetadata);

    }

    /**
     * Apply rules for a common bean (message-destination-ref).
     * @param messageDestinationRefList the list of message-destination-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    private void applyMessageDestinationRef(final List<MessageDestinationRef> messageDestinationRefList,
            final IEjbJarClassMetadata<?> classAnnotationMetadata) {
        if (messageDestinationRefList == null) {
            return;
        }

        // For each message-destination-ref, build a &#64;{@link
        // javax.annotation.Resource}
        for (MessageDestinationRef messageDestinationRef : messageDestinationRefList) {
            JAnnotationResource jResource = new JAnnotationResource();
            // set name
            jResource.setName(messageDestinationRef.getName());

            // set type
            jResource.setType(messageDestinationRef.getType());

            // Mapped Name
            jResource.setMappedName(messageDestinationRef.getMappedName());

            // Apply the resource on the class
            applyJResource(jResource, classAnnotationMetadata, messageDestinationRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (resource-ref).
     * @param resourceRefList the list of resource-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    private void applyResourceRef(final List<ResourceRef> resourceRefList, final IEjbJarClassMetadata<?> classAnnotationMetadata) {
        if (resourceRefList == null) {
            return;
        }

        // For each resource-ref, build a &#64;{@link javax.annotation.Resource}
        for (ResourceRef resourceRef : resourceRefList) {
            JAnnotationResource jResource = new JAnnotationResource();
            // set name
            jResource.setName(resourceRef.getResRefName());

            // set type
            jResource.setType(resourceRef.getResRefType());

            // Mapped Name
            jResource.setMappedName(resourceRef.getMappedName());

            // Apply the resource on the class
            applyJResource(jResource, classAnnotationMetadata, resourceRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (resource-env-ref).
     * @param resourceEnvRefList the list of resource-env-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    private void applyResourceEnvRef(final List<ResourceEnvRef> resourceEnvRefList,
            final IEjbJarClassMetadata<?> classAnnotationMetadata) {
        if (resourceEnvRefList == null) {
            return;
        }

        // For each resource-ref, build a &#64;{@link javax.annotation.Resource}
        for (ResourceEnvRef resourceEnvRef : resourceEnvRefList) {
            JAnnotationResource jResource = new JAnnotationResource();
            // set name
            jResource.setName(resourceEnvRef.getResourceEnvRefName());

            // set type
            jResource.setType(resourceEnvRef.getResourceEnvRefType());

            // Mapped Name
            jResource.setMappedName(resourceEnvRef.getMappedName());

            // Apply the resource on the class
            applyJResource(jResource, classAnnotationMetadata, resourceEnvRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (ejb-ref).
     * @param ejbRefList the list of ejb-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    private void applyEjbRef(final List<EJBRef> ejbRefList, final IEjbJarClassMetadata<?> classAnnotationMetadata) {
        if (ejbRefList == null) {
            return;
        }

        // For each ejb-ref, build a &#64;{@link javax.ejb.EJB}
        for (EJBRef ejbRef : ejbRefList) {
            JEjbEJB jEJB = buildEJB(ejbRef);
            // set interface
            if (ejbRef.getHome() != null) {
                jEJB.setBeanInterface(ejbRef.getHome());
            } else if (ejbRef.getRemote() != null) {
                jEJB.setBeanInterface(ejbRef.getRemote());
            }
            applyJEJB(jEJB, classAnnotationMetadata, ejbRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (ejb-local-ref).
     * @param ejbLocalRefList the list of ejb-local-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    private void applyEjbLocalRef(final List<EJBLocalRef> ejbLocalRefList, final IEjbJarClassMetadata<?> classAnnotationMetadata) {
        if (ejbLocalRefList == null) {
            return;
        }

        // For each ejb-local-ref, build a &#64;{@link javax.ejb.EJB}
        for (EJBLocalRef ejbLocalRef : ejbLocalRefList) {
            JEjbEJB jEJB = buildEJB(ejbLocalRef);
            // set interface
            if (ejbLocalRef.getLocalHome() != null) {
                jEJB.setBeanInterface(ejbLocalRef.getLocalHome());
            } else if (ejbLocalRef.getLocal() != null) {
                jEJB.setBeanInterface(ejbLocalRef.getLocal());
            }
            applyJEJB(jEJB, classAnnotationMetadata, ejbLocalRef.getInjectionTargetList());
        }
    }

    /**
     * Adds the given &#64;{@link javax.annotation.Resource} object to the
     * class and apply it also on injection target elements.
     * @param jResource an &#64;{@link javax.annotation.Resource} impl
     * @param classAnnotationMetadata the bean's class
     * @param injectionTargetList the list of injection fields to set
     */
    private static void applyJResource(final JAnnotationResource jResource,
            final IEjbJarClassMetadata<?> classAnnotationMetadata, final List<InjectionTarget> injectionTargetList) {

        // Add field on the class
        IJAnnotationResource existingJResource = classAnnotationMetadata.getJAnnotationResource();

        // Needs to use a list instead.
        List<IJAnnotationResource> jResources = classAnnotationMetadata.getJAnnotationResources();
        if (jResources == null) {
            jResources = new ArrayList<IJAnnotationResource>();
            classAnnotationMetadata.setJAnnotationResources(jResources);
        }

        // put existing element in the list
        if (existingJResource != null) {
            jResources.add(existingJResource);
            // remove single element
            classAnnotationMetadata.setJAnnotationResource(null);
        }

        // Add our new element
        jResources.add(jResource);

        // Now, add elements on the injection fields
        if (injectionTargetList != null) {
            for (InjectionTarget injectionTarget : injectionTargetList) {

                // Add the JAnnotationResource on it
                getInjectionTarget(classAnnotationMetadata, injectionTarget).setJAnnotationResource(jResource);
            }
        }

    }

    /**
     * Gets the metadata for a given injection target.
     * @param classAnnotationMetadata the class used to search the target
     *        element
     * @param injectionTarget the given target info element
     * @return a common annotation metadata if element is found
     */
    private static ISharedMetadata getInjectionTarget(final IEjbJarClassMetadata<?> classAnnotationMetadata,
            final InjectionTarget injectionTarget) {
        String injectionTargetName = injectionTarget.getTargetName();

        // First, try with the method
        IEjbJarMethodMetadata<?> method = getMethod(classAnnotationMetadata, injectionTarget);

        // found ? return it
        if (method != null) {
            return method;
        }

        // else, try with the next item which should be a field
        // Try with the field
        IEjbJarFieldMetadata<?> field = getField(classAnnotationMetadata, injectionTarget);
        if (field == null) {
            throw new IllegalArgumentException("No method or field for injection target name '" + injectionTargetName
                    + "' found in the class '" + classAnnotationMetadata.getClassName() + "'.");
        }

        return field;

    }

    /**
     * Gets the method metadata for a given injection target.
     * @param classAnnotationMetadata the class used to search the target
     *        element
     * @param injectionTarget the given target info element
     * @return a method metadata if element is found
     */
    private static IEjbJarMethodMetadata<?> getMethod(final IEjbJarClassMetadata<?> classAnnotationMetadata,
            final InjectionTarget injectionTarget) {
        // get data from the injection target element
        String clName = injectionTarget.getClassname();
        if (clName == null) {
            clName = classAnnotationMetadata.getClassName();
        } else {
            clName = clName.replace(".", "/");
        }

        // Get method name which is the injection target name
        String methodName = injectionTarget.getTargetName();

        // Now, search the element in the given class
        IEjbJarClassMetadata<?> foundClassMetadata = classAnnotationMetadata.getEjbJarDeployableMetadata()
                .getEjbJarClassMetadata(clName);
        if (foundClassMetadata == null) {
            throw new IllegalArgumentException("Cannot find the class metadata named '" + clName + "'.");
        }

        // Got the class metadata, now search the field
        List<? extends IEjbJarMethodMetadata<?>> methods = foundClassMetadata.searchStandardMethodMetadata(methodName);

        if (methods.size() == 0) {
            // no method
            return null;
        } else if (methods.size() > 1) {
            // Should be only one
            throw new IllegalArgumentException("Too many methods with name '" + methodName + "' found in the class '" + clName
                    + "'.");
        }

        return methods.get(0);
    }

    /**
     * Gets the field metadata for a given injection target.
     * @param classAnnotationMetadata the class used to search the target
     *        element
     * @param injectionTarget the given target info element
     * @return a field if element is found
     */
    private static IEjbJarFieldMetadata<?> getField(final IEjbJarClassMetadata<?> classAnnotationMetadata,
            final InjectionTarget injectionTarget) {
        // get data from the injection target element
        String clName = injectionTarget.getClassname();
        if (clName == null) {
            clName = classAnnotationMetadata.getClassName();
        } else {
            clName = clName.replace(".", "/");
        }

        String fieldName = injectionTarget.getTargetName();

        // Now, search the element in the given class
        IEjbJarClassMetadata<?> foundClassMetadata = classAnnotationMetadata.getEjbJarDeployableMetadata()
                .getEjbJarClassMetadata(clName);
        if (foundClassMetadata == null) {
            throw new IllegalArgumentException("Cannot find the class metadata named '" + clName + "'.");
        }

        // Got the class metadata, now search the field
        List<? extends IEjbJarFieldMetadata<?>> fields = foundClassMetadata.searchStandardFieldMetadata(fieldName);

        if (fields.size() == 0) {
            // no field
            return null;
        } else if (fields.size() > 1) {
            // Should be only one
            throw new IllegalArgumentException("Too many fields with name '" + fieldName + "' found in the class '" + clName
                    + "'.");
        }

        return fields.get(0);
    }

    /**
     * Adds the given &#64;{@link javax.ejb.EJB} object to the class and apply
     * it also on injection target elements.
     * @param jEJB an &#64;{@link javax.ejb.EJB} impl
     * @param classAnnotationMetadata the bean's class
     * @param injectionTargetList the list of injection fields to set
     */
    private static void applyJEJB(final JEjbEJB jEJB, final IEjbJarClassMetadata<?> classAnnotationMetadata,
            final List<InjectionTarget> injectionTargetList) {

        // Add field on the class
        IJEjbEJB existingJEjbEJB = classAnnotationMetadata.getJEjbEJB();

        // Needs to use a list instead.
        List<IJEjbEJB> jEjbEJBs = classAnnotationMetadata.getJEjbEJBs();
        if (jEjbEJBs == null) {
            jEjbEJBs = new ArrayList<IJEjbEJB>();
            classAnnotationMetadata.setJEjbEJBs(jEjbEJBs);
        }

        // put existing element in the list
        if (existingJEjbEJB != null) {
            jEjbEJBs.add(existingJEjbEJB);
            // remove single element
            classAnnotationMetadata.setJEjbEJB(null);
        }

        // Add our new element
        jEjbEJBs.add(jEJB);

        // Now, add elements on the injection fields
        if (injectionTargetList != null) {
            for (InjectionTarget injectionTarget : injectionTargetList) {

                // Add the JEjbEJB on it
                getInjectionTarget(classAnnotationMetadata, injectionTarget).setJEjbEJB(jEJB);
            }
        }

    }

    /**
     * Build an &#64;{@link javax.ejb.EJB} impl for the given
     * ejb-ref/ejb-local-ref.
     * @param ref an instance of a ejb-ref/ejb-local-ref.
     * @return a implementation of &#64;{@link javax.ejb.EJB} interface.
     */
    private static JEjbEJB buildEJB(final AbsEJBRef ref) {
        JEjbEJB jEJB = new JEjbEJB();
        // Sets the name
        if (ref.getEjbRefName() != null) {
            jEJB.setName(ref.getEjbRefName());
        }

        // sets the mapped-name
        if (ref.getMappedName() != null) {
            jEJB.setMappedName(ref.getMappedName());
        }

        // sets the bean-name
        if (ref.getEjbLink() != null) {
            jEJB.setBeanName(ref.getEjbLink());
        }

        return jEJB;
    }

    /**
     * Apply rules for a common bean (method callbacks).
     * @param aroundInvokeList the list of method/aroundInvoke callbacks.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     * @param type the type of lifecycle.
     */
    private void applyAroundInvoke(final List<AroundInvoke> aroundInvokeList,
            final IEjbJarClassMetadata<E> classAnnotationMetadata, final String type) {
        for (AroundInvoke aroundInvoke : aroundInvokeList) {
            String interceptorClassName = null;

            // if no class, takes the bean's class
            if (aroundInvoke.getClassName() == null) {
                interceptorClassName = classAnnotationMetadata.getClassName();
            } else {
                interceptorClassName = encode(aroundInvoke.getClassName());
            }

            // Now, search the Method on the class or its super-class if not
            // found
            String searchMethodName = aroundInvoke.getMethodName();
            if (searchMethodName == null) {
                throw new IllegalStateException("Invalid null method name for the callback '" + interceptorClassName + "'.");
            }

            // Apply interceptor
            applyInterceptor(type, interceptorClassName, searchMethodName, classAnnotationMetadata);

        }
    }

    /**
     * Apply rules for a common bean (lifecycle callbacks).
     * @param lifecycleList the list of lifecycle callbacks.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     * @param type the type of lifecycle.
     */
    private void applyLifeCycle(final List<LifeCycleCallback> lifecycleList,
            final IEjbJarClassMetadata<E> classAnnotationMetadata, final String type) {
        for (LifeCycleCallback lifecycleCallback : lifecycleList) {

            String interceptorClassName = null;

            // if no class, takes the bean's class
            if (lifecycleCallback.getLifecycleCallbackClass() == null) {
                interceptorClassName = classAnnotationMetadata.getClassName();
            } else {
                interceptorClassName = encode(lifecycleCallback.getLifecycleCallbackClass());
            }

            // Now, search the Method on the class or its super-class if not
            // found
            String searchMethodName = lifecycleCallback.getLifecycleCallbackMethod();
            if (searchMethodName == null) {
                throw new IllegalStateException("Invalid null method name for the callback '" + interceptorClassName + "'.");
            }

            // Apply interceptor
            applyInterceptor(type, interceptorClassName, searchMethodName, classAnnotationMetadata);

        }
    }

    /**
     * Adds a link on the given class to the interceptor class.
     * @param classAnnotationMetadata the class that will use/import the
     *        interceptor
     * @param interceptorClassName the interceptor class.
     */
    private void addInterceptorForClass(final IEjbJarClassMetadata<?> classAnnotationMetadata, final String interceptorClassName) {
        // Get interceptors for the given class (the bean class)
        IJInterceptors interceptors = classAnnotationMetadata.getAnnotationInterceptors();
        if (interceptors == null) {
            // build new list if empty
            interceptors = new JInterceptors();
            classAnnotationMetadata.setAnnotationsInterceptors(interceptors);
        }

        // class already present ?
        if (!interceptors.contains(interceptorClassName)) {
            interceptors.addClass(interceptorClassName);
        }
    }

    /**
     * Finds a method in the interceptor class and set the correct type value.
     * @param type type of interceptor
     * @param interceptorClassName the name of the interceptor class
     * @param methodName the name of the method in the interceptor class
     * @param classAnnotationMetadata the class that is using the interceptor
     *        (may be null)
     */
    private void applyInterceptor(final String type, final String interceptorClassName, final String methodName,
            final IEjbJarClassMetadata<E> classAnnotationMetadata) {

        // if the bean is using this interceptor and that this is not the bean
        // class, add the link.
        if (classAnnotationMetadata != null && !classAnnotationMetadata.getClassName().equals(interceptorClassName)) {
            addInterceptorForClass(classAnnotationMetadata, interceptorClassName);
        }

        IEjbJarMethodMetadata<E> foundMethod = null;
        IEjbJarClassMetadata<E> interceptorMetadata = getEjbJarDeployableMetadata()
                .getEjbJarClassMetadata(interceptorClassName);

        boolean loop = true;
        // While method is not found and that class has super class, try to
        // search
        while (foundMethod == null && loop) {
            List<? extends IEjbJarMethodMetadata<E>> listMethods = interceptorMetadata.searchStandardMethodMetadata(methodName);

            // Method present ?
            if (listMethods.size() == 1) {
                // first item
                foundMethod = listMethods.get(0);
                continue;
            } else if (listMethods.size() > 1) {
                throw new IllegalStateException("Method with name '" + methodName + "' was found more than once on the class '"
                        + interceptorMetadata.getClassName() + "'.");
            }

            // else continue
            String superClassName = interceptorMetadata.getSuperName();
            if (!superClassName.equals(JAVA_LANG_OBJECT)) {
                interceptorMetadata = getEjbJarDeployableMetadata().getEjbJarClassMetadata(superClassName);
            } else {
                // break loop
                loop = false;
            }
        }

        if (foundMethod == null) {
            throw new IllegalStateException("The method named '" + methodName + "' was not found in the class '"
                    + interceptorMetadata.getClassName() + "' and all its super-classes.");
        }

        // call setter to true and only if it is not already present.
        if (POST_CONSTRUCT.equals(type)) {
            if (!foundMethod.isPostConstruct()) {
                foundMethod.setPostConstruct(true);
             // TODO gaellalire : move to ezb -> interceptorMetadata.addPostConstructMethodMetadata(foundMethod);
            }
        } else if (PRE_DESTROY.equals(type)) {
            if (!foundMethod.isPreDestroy()) {
                foundMethod.setPreDestroy(true);
             // TODO gaellalire : move to ezb -> interceptorMetadata.addPreDestroyMethodMetadata(foundMethod);
            }
        } else if (PRE_PASSIVATE.equals(type)) {
            if (!foundMethod.isPrePassivate()) {
                foundMethod.setPrePassivate(true);
             // TODO gaellalire : move to ezb -> interceptorMetadata.addPrePassivateMethodMetadata(foundMethod);
            }
        } else if (POST_ACTIVATE.equals(type)) {
            if (!foundMethod.isPostActivate()) {
                foundMethod.setPostActivate(true);
             // TODO gaellalire : move to ezb -> interceptorMetadata.addPostActivateMethodMetadata(foundMethod);
            }
        } else if (AroundInvoke.NAME.equals(type)) {
            if (!foundMethod.isAroundInvoke()) {
                foundMethod.setAroundInvoke(true);
             // TODO gaellalire : move to ezb -> interceptorMetadata.addAroundInvokeMethodMetadata(foundMethod);
            }
        } else {
            throw new IllegalStateException("No case for type '" + type + "'..");
        }
    }

    /**
     * Set/override the current application-exception.
     * @param assemblyDescriptor the given assembly descriptor to analyze.
     */
    public void mergeApplicationException(final AssemblyDescriptor assemblyDescriptor) {
        List<JApplicationException> assemblyDescriptorList = assemblyDescriptor.getApplicationExceptionList();
        if (assemblyDescriptorList != null) {
            for (JApplicationException applicationException : assemblyDescriptorList) {
                String className = applicationException.getClassName();
                // Get metadata for the given class
                IEjbJarClassMetadata<E> exceptionMetadata = getEjbJarDeployableMetadata().getEjbJarClassMetadata(
                        encode(className));
                if (exceptionMetadata == null) {
                    throw new IllegalStateException("The class named '" + className
                            + "' was not found. Cannot set ApplicationException on it");
                }
                // Set the new value
                exceptionMetadata.setApplicationException(applicationException);
                logger.debug("Setting the application-exception ''{0}'' on the class ''{1}''", applicationException, className);
            }
        }
    }

    /**
     * Set/override the current security.
     * @param assemblyDescriptor the given assembly descriptor to analyze.
     */
    public void mergeAssemblySecurity(final AssemblyDescriptor assemblyDescriptor) {

        // Search the security role
        List<String> securityRoles = assemblyDescriptor.getSecurityRoleList();
        for (String role : securityRoles) {
            // Gets the metadata for all ejb
            for (IEjbJarClassMetadata<?> beanMetadata : getEjbJarDeployableMetadata().getEjbJarClassMetadataCollection()) {
                if (beanMetadata.isBean()) {
                    List<String> declaredRoles = beanMetadata.getDeclareRoles();
                    if (declaredRoles == null) {
                        declaredRoles = new ArrayList<String>();
                    }
                    if (!declaredRoles.contains(role)) {
                        declaredRoles.add(role);
                        beanMetadata.setDeclareRoles(declaredRoles);
                    }
                }
            }
        }

        // sets the per-role or unchecked methods
        List<MethodPermission> methodPermissionList = assemblyDescriptor.getMethodPermissionList();
        for (MethodPermission methodPermission : methodPermissionList) {

            List<MethodDD> methodDDs = methodPermission.getMethods();
            for (MethodDD method : methodDDs) {

                // Get Ejb-Name
                String ejbName = method.getEjbName();

                // Gets the metadata for the ejb
                IEjbJarClassMetadata<?> beanMetadata = getEjbJarDeployableMetadata().getEjbJarClassMetadataForEjbName(ejbName);
                if (beanMetadata == null) {
                    throw new IllegalStateException("No metadata found for the the ejb '" + ejbName
                            + "'. Check that this ejb is defined. This ejb-name is referenced from "
                            + "a method of an assembly descriptor / method-permission");
                }

                // Get matching methods
                List<? extends IEjbJarMethodMetadata<?>> methods = getMethodsForGivenMethodDD(method, beanMetadata);

                for (IEjbJarMethodMetadata<?> methodAnnotationMetadata : methods) {
                    // Role -base or unchecked ?
                    if (methodPermission.isUnchecked()) {
                        methodAnnotationMetadata.setPermitAll(true);
                    } else if (methodPermission.getRoleNameList().size() > 0) {
                        for (String role : methodPermission.getRoleNameList()) {
                            List<String> existingRolesAllowed = methodAnnotationMetadata.getRolesAllowed();
                            if (existingRolesAllowed == null) {
                                existingRolesAllowed = new ArrayList<String>();
                            }
                            if (!existingRolesAllowed.contains(role)) {
                                existingRolesAllowed.add(role);
                                methodAnnotationMetadata.setRolesAllowed(existingRolesAllowed);
                            }
                        }
                    }
                }
            }
        }

        // Exclude the given methods of the XML DD
        List<MethodDD> excludeMethodList = assemblyDescriptor.getExcludeListMethods();
        for (MethodDD method : excludeMethodList) {

            // Get Ejb-Name
            String ejbName = method.getEjbName();

            // Gets the metadata for the ejb
            IEjbJarClassMetadata<?> beanMetadata = getEjbJarDeployableMetadata().getEjbJarClassMetadataForEjbName(ejbName);
            if (beanMetadata == null) {
                throw new IllegalStateException("No metadata found for the the ejb '" + ejbName
                        + "'. Check that this ejb is defined. This ejb-name is referenced from "
                        + "a method of an assembly descriptor / exclude-list");
            }

            // Get matching methods
            List<? extends IEjbJarMethodMetadata<?>> methods = getMethodsForGivenMethodDD(method, beanMetadata);

            // Set the deny all flag to these methods
            for (IEjbJarMethodMetadata<?> methodAnnotationMetadata : methods) {
                methodAnnotationMetadata.setDenyAll(true);
            }
        }
    }

    /**
     * Set/override the current container-transaction.
     * @param assemblyDescriptor the given assembly descriptor to analyze.
     */
    public void mergeContainerTransaction(final AssemblyDescriptor assemblyDescriptor) {
        List<ContainerTransaction> containerTransactionList = assemblyDescriptor.getContainerTransactionList();
        if (containerTransactionList == null) {
            return;
        }

        for (ContainerTransaction containerTransaction : containerTransactionList) {

            // No transaction attribute ? then escape
            TransactionAttributeType transactionAttributeType = containerTransaction.getTransactionAttribute();
            if (transactionAttributeType == null) {
                continue;
            }

            for (MethodDD method : containerTransaction.getMethods()) {
                // Get Ejb-Name
                String ejbName = method.getEjbName();

                // Gets the metadata for the ejb
                IEjbJarClassMetadata<?> beanMetadata = getEjbJarDeployableMetadata().getEjbJarClassMetadataForEjbName(ejbName);
                if (beanMetadata == null) {
                    throw new IllegalStateException("No metadata found for the the ejb '" + ejbName
                            + "'. Check that this ejb is defined. This ejb-name is referenced from "
                            + "a method of a container transaction");
                }

                // Get Ejb-Name
                String methodName = method.getName();
                TransactionAttributeLevel txlevel = TransactionAttributeLevel.XML_WILDCARD;

                // Style 1 (see �13.3.7.2)
                if (WILDCARD.equals(methodName)) {
                    // Apply tx for all methods if it can be applied
                    for (IEjbJarMethodMetadata<?> methodAnnotationMetadata : beanMetadata.getStandardMethodMetadataCollection()) {
                        // check level ?
                        if (methodAnnotationMetadata.getTransactionAttributeLevel() == TransactionAttributeLevel.ANNOTATION) {
                            methodAnnotationMetadata.setTransactionAttributeType(transactionAttributeType);
                            methodAnnotationMetadata.setTransactionAttributeLevel(txlevel);
                            logger.debug("Setting transaction attribute of method {0} of bean {1} to {2}",
                                    methodAnnotationMetadata, ejbName, transactionAttributeType);
                        }

                    }
                    // next step
                    continue;
                }

                // Style 2 ?
                List<String> params = method.getParams();

                if (params.size() == 0) {
                    txlevel = TransactionAttributeLevel.XML_METHOD_NAME;
                } else {
                    txlevel = TransactionAttributeLevel.XML_METHOD_PARAMS;
                }

                // Get matching methods
                List<? extends IEjbJarMethodMetadata<?>> methods = getMethodsForGivenMethodDD(method, beanMetadata);
                if (methods.size() == 0) {
                    throw new IllegalStateException("No matching method found with '" + method + "' for bean '" + ejbName
                            + "'.");
                }

                for (IEjbJarMethodMetadata<?> methodAnnotationMetadata : methods) {

                    // check level ?
                    if (methodAnnotationMetadata.getTransactionAttributeLevel() == TransactionAttributeLevel.ANNOTATION
                            || methodAnnotationMetadata.getTransactionAttributeLevel() == TransactionAttributeLevel.XML_WILDCARD) {

                        if ((txlevel == TransactionAttributeLevel.XML_METHOD_NAME && methodAnnotationMetadata
                                .getTransactionAttributeLevel() != TransactionAttributeLevel.XML_METHOD_PARAMS)
                                || (txlevel == TransactionAttributeLevel.XML_METHOD_PARAMS)) {
                            methodAnnotationMetadata.setTransactionAttributeType(transactionAttributeType);
                            methodAnnotationMetadata.setTransactionAttributeLevel(txlevel);
                            logger.debug("Setting transaction attribute of method {0} of bean {1} to {2}",
                                    methodAnnotationMetadata, ejbName, txlevel);
                        }
                    }

                }
            }
        }
    }

    /**
     * Set/override the current interceptor-binding.
     * @param assemblyDescriptor the given assembly descriptor to analyze.
     */
    public void mergeInterceptorBinding(final AssemblyDescriptor assemblyDescriptor) {
        List<InterceptorBinding> interceptorBindingList = assemblyDescriptor.getInterceptorBindingList();
        if (interceptorBindingList != null) {
            for (InterceptorBinding interceptorBinding : interceptorBindingList) {

                // Get Ejb-Name
                String ejbName = interceptorBinding.getEjbName();
                // Default interceptor
                if (WILDCARD.equals(ejbName)) {
                    // Get current list
                    IJInterceptors defaultInterceptors = getEjbJarDeployableMetadata().getDefaultInterceptorsClasses();
                    if (defaultInterceptors == null) {
                        // build new list if empty
                        defaultInterceptors = new JInterceptors();
                        getEjbJarDeployableMetadata().setDefaultInterceptorsClasses(defaultInterceptors);
                    }

                    // Add the interceptors of the interceptor-binding to the
                    // default interceptors
                    for (String interceptor : interceptorBinding.getInterceptorClassList()) {
                        defaultInterceptors.addClass(encode(interceptor));
                    }

                    // stop the loop
                    continue;
                }

                // Gets the metadata for the ejb
                IEjbJarClassMetadata<?> beanMetadata = getEjbJarDeployableMetadata().getEjbJarClassMetadataForEjbName(ejbName);
                if (beanMetadata == null) {
                    throw new IllegalStateException("No metadata found for the the ejb '" + ejbName
                            + "'. Check that this ejb is defined. This ejb-name is referenced from an interceptor-binding");
                }

                // It is a method ?
                MethodDD methodDD = interceptorBinding.getMethod();

                // not a method
                if (methodDD == null) {
                    IJInterceptors interceptors = beanMetadata.getAnnotationInterceptors();
                    if (interceptors == null) {
                        // build new list if empty
                        interceptors = new JInterceptors();
                        beanMetadata.setAnnotationsInterceptors(interceptors);
                    }

                    for (String interceptor : interceptorBinding.getInterceptorClassList()) {
                        interceptors.addClass(encode(interceptor));
                    }

                    // Exclude default interceptors for this bean ?
                    if (interceptorBinding.isExcludeDefaultInterceptorsCalled()) {
                        beanMetadata.setExcludeDefaultInterceptors(interceptorBinding.isExcludeDefaultInterceptors());
                    }

                    // For ordered item: First, add interceptors (if they are
                    // not already present)
                    List<String> orderList = interceptorBinding.getOrderInterceptorClassList();
                    if (orderList != null) {

                        // Check that this is the first time that we try to
                        // order the list.
                        if (beanMetadata.isOrderedInterceptors()) {
                            throw new IllegalStateException("The interceptor-order element has already been used on the bean '"
                                    + ejbName + "'. It can only be used once.");
                        }

                        // Flag class as a class having ordered interceptors
                        beanMetadata.setOrderedInterceptors(true);

                        // Build a list with all interceptors. (it means that we
                        // add default interceptors if any) if they are not
                        // excluded
                        if (!beanMetadata.isExcludedDefaultInterceptors()) {
                            if (beanMetadata.getEjbJarDeployableMetadata().getDefaultInterceptorsClasses() != null) {
                                for (String cls : beanMetadata.getEjbJarDeployableMetadata().getDefaultInterceptorsClasses()
                                        .getClasses()) {
                                    interceptors.addClass(cls);
                                }
                            }
                        }

                        // Then add new items found in the DD
                        for (String interceptor : interceptorBinding.getOrderInterceptorClassList()) {
                            String encodedName = encode(interceptor);
                            if (!interceptors.contains(encodedName)) {
                                interceptors.addClass(encodedName);
                            }
                        }

                        // Order.
                        // First, check that the list has the correct size as it
                        // should be a total ordering.
                        if (interceptors.size() != orderList.size()) {
                            String err = "The list used for ordering interceptors is not a total list as the size mismatch.";
                            err += "The current list is '" + interceptors + "' while the ordering list is '" + orderList + "'.";
                            throw new IllegalStateException(err);
                        }

                        // Size is ok, check that all elements are present
                        for (String clazz : orderList) {
                            if (!interceptors.contains(encode(clazz))) {
                                throw new IllegalStateException("The element '" + clazz
                                        + "' of the ordered list is not present in the interceptors '" + interceptors + "'.");
                            }
                        }

                        // Ok, all the elements are present, set the new
                        // ordering.
                        // Which is based on the given ordered list.
                        JInterceptors orderedInterceptors = new JInterceptors();
                        for (String clazz : orderList) {
                            orderedInterceptors.addClass(encode(clazz));
                        }
                        // Set the new list
                        beanMetadata.setAnnotationsInterceptors(orderedInterceptors);

                    }
                } else {
                    // method case
                    List<? extends IEjbJarMethodMetadata<?>> methods = getMethodsForGivenMethodDD(methodDD, beanMetadata);

                    // If no elements, print warning
                    if (methods.size() == 0) {
                        logger.warn("Method with name '" + methodDD.getName()
                                + "' is specified in interceptorBinding element for class '" + beanMetadata.getClassName()
                                + "' but no methods are matching");
                    }

                    // now, apply interceptor on each method
                    for (IEjbJarMethodMetadata<?> method : methods) {
                        IJInterceptors interceptors = method.getAnnotationInterceptors();
                        if (interceptors == null) {
                            // build new list if empty
                            interceptors = new JInterceptors();
                            method.setAnnotationsInterceptors(interceptors);
                        }

                        for (String interceptor : interceptorBinding.getInterceptorClassList()) {
                            interceptors.addClass(encode(interceptor));
                        }

                        // Exclude class interceptors for this method ?
                        if (interceptorBinding.isExcludeClassInterceptorsCalled()) {
                            method.setExcludeClassInterceptors(interceptorBinding.isExcludeClassInterceptors());
                        }
                        // Exclude default interceptors for this method ?
                        if (interceptorBinding.isExcludeDefaultInterceptorsCalled()) {
                            method.setExcludeDefaultInterceptors(interceptorBinding.isExcludeDefaultInterceptors());
                        }

                    }
                }
            }
        }
    }

    /**
     * Gets the methods metadata that are matching a given method DD.
     * @param methodDD the methodDD object
     * @param classAnnotationMetadata the class where to search methods.
     * @return the list of methods metadata
     */
    private static List<? extends IEjbJarMethodMetadata<?>> getMethodsForGivenMethodDD(final MethodDD methodDD,
            final IEjbJarClassMetadata<?> classAnnotationMetadata) {
        // get the method.
        String methodName = methodDD.getName();
        List<? extends IEjbJarMethodMetadata<?>> methods = classAnnotationMetadata.searchStandardMethodMetadata(methodName);
        // params ?
        List<String> params = methodDD.getParams();
        // Keep only method with the correct params
        if (params != null && params.size() > 0) {
            List<IEjbJarMethodMetadata<?>> newMethods = new ArrayList<IEjbJarMethodMetadata<?>>();
            for (IEjbJarMethodMetadata<?> method : methods) {
                // get params for the current method
                Type[] types = Type.getArgumentTypes(method.getJMethod().getDescriptor());
                // Same size ?
                if (params.size() != types.length) {
                    // Not same size, go to the next one
                    continue;
                }

                // Compare each element
                int i = 0;
                boolean matching = false;
                for (String param : params) {
                    if (param.equals(types[i].getClassName())) {
                        matching = true;
                    }
                    i++;
                }
                // match ?
                if (matching) {
                    newMethods.add(method);
                }
            }
            // Get the new list (only matching elements)
            methods = newMethods;
        }
        return methods;
    }

}
