/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RemoveMethod.java 3319 2008-05-27 13:00:31Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.struct;

import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.MethodDD;

/**
 * This class manages the &lt;remove-method&gt; element of the xml DD.
 * @author Florent Benoit
 */
public class RemoveMethod {

    /**
     * Name of this element.
     */
    public static final String NAME = "remove-method";

    /**
     * This remove method is linked to a Method of the Bean (with params).
     */
    private MethodDD method = null;

    /**
     * boolean used to know if when there is an exception, the bean should be kept or not.
     * It uses a boolean object to know if the element has been set or not
     */
    private Boolean retainIfException = null;


    /**
     * Sets the method of the bean which needs to be marked.
     * @param method the method to use
     */
    public void setMethod(final MethodDD method) {
        this.method = method;
    }

    /**
     * @return the method that will be used as a remove method
     */
    public MethodDD getMethod() {
        return method;
    }

    /**
     * If there is an exception, the bean should be kept or not.
     * @param retainIfException true to keep, or false to remove
     */
    public void setRetainIfException(final boolean retainIfException) {
        this.retainIfException = Boolean.valueOf(retainIfException);
    }

    /**
     * @return the flag to know if when there is an exception, the bean should be kept or not.
     * It returns null if it was not set.
     */
    public Boolean getRetainIfException() {
        return retainIfException;
    }

}
